@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 Base class for control volumes
 """
@@ -199,7 +199,7 @@ CONFIG_Template.declare("has_work_transfer", ConfigValue(
 CONFIG_Template.declare("has_enthalpy_transfer", ConfigValue(
     default=False,
     domain=In([True, False]),
-    description="Entahlpy transfer term construction flag",
+    description="Enthalpy transfer term construction flag",
     doc="""Indicates whether terms for enthalpy transfer due to mass trasnfer
 should be constructed, **default** - False.
 **Valid values** {
@@ -421,6 +421,7 @@ have a config block which derives from CONFIG_Base,
                         "your property package to implement the necessary "
                         "default attributes.".format(self.name))
 
+        self._constructed_material_balance_type = balance_type
         if balance_type == MaterialBalanceType.none:
             mb = None
         elif balance_type == MaterialBalanceType.componentPhase:
@@ -456,6 +457,8 @@ have a config block which derives from CONFIG_Base,
                 should be included in energy balances
             has_work_transfer (bool): whether generic mass transfer terms
                 should be included in energy balances
+            has_enthalpy_transfer (bool): whether generic enthalpy transfer
+                terms should be included in energy balances
             custom_term (Expression): a Pyomo Expression representing custom
                 terms to be included in energy balances
 
@@ -476,6 +479,7 @@ have a config block which derives from CONFIG_Base,
                         "your property package to implement the necessary "
                         "default attributes.".format(self.name))
 
+        self._constructed_energy_balance_type = balance_type
         if balance_type == EnergyBalanceType.none:
             eb = None
         elif balance_type == EnergyBalanceType.enthalpyTotal:
@@ -514,6 +518,7 @@ have a config block which derives from CONFIG_Base,
         Returns:
             Constraint objects constructed by sub-method
         """
+        self._constructed_momentum_balance_type = balance_type
         if balance_type == MomentumBalanceType.none:
             mb = None
         elif balance_type == MomentumBalanceType.pressureTotal:
@@ -795,10 +800,10 @@ have a config block which derives from CONFIG_Base,
                 return 1
             elif (flow_basis == MaterialFlowBasis.mass and
                   rxn_basis == MaterialFlowBasis.molar):
-                return prop.mw[j]
+                return prop.mw_comp[j]
             elif (flow_basis == MaterialFlowBasis.molar and
                   rxn_basis == MaterialFlowBasis.mass):
-                return 1/prop.mw[j]
+                return 1/prop.mw_comp[j]
             else:
                 raise BurntToast(
                         "{} encountered unrecognsied combination of bases "
@@ -813,7 +818,7 @@ have a config block which derives from CONFIG_Base,
 
     def _get_representative_property_block(self):
         try:
-            t_ref = self.flowsheet().config.time.first()
+            t_ref = self.flowsheet().time.first()
         except AttributeError:
             raise ConfigurationError(
                     "{} control volume does not appear to be part of a "
