@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 This module contains classes for property blocks and property parameter blocks.
 """
@@ -17,11 +17,11 @@ This module contains classes for property blocks and property parameter blocks.
 import sys
 
 # Import Pyomo libraries
-from pyomo.environ import Set, value
+from pyomo.environ import Set, value, Var, Expression, Constraint
 from pyomo.core.base.var import _VarData
 from pyomo.core.base.expression import _ExpressionData
 from pyomo.common.config import ConfigBlock, ConfigValue, In
-from pyomo.core.base.misc import tabular_writer
+from pyomo.common.formatting import tabular_writer
 
 # Import IDAES cores
 from idaes.core.process_block import ProcessBlock
@@ -32,6 +32,7 @@ from idaes.core.phases import Phase, PhaseData
 from idaes.core.components import Component, ComponentData
 from idaes.core.util.config import is_physical_parameter_block
 from idaes.core.util.exceptions import (BurntToast,
+                                        ConfigurationError,
                                         PropertyNotSupportedError,
                                         PropertyPackageError)
 from idaes.core.util.misc import add_object_reference
@@ -39,6 +40,7 @@ from idaes.core.util.model_statistics import (degrees_of_freedom,
                                               number_variables,
                                               number_activated_constraints,
                                               number_activated_blocks)
+from idaes.core.util import scaling as iscale
 import idaes.logger as idaeslog
 
 # Some more information about this module
@@ -52,6 +54,16 @@ __all__ = ['StateBlockData',
 _log = idaeslog.getLogger(__name__)
 
 
+class _lock_attribute_creation_context(object):
+    """Context manager to lock creation of new attributes on a state block"""
+    def __init__(self, block):
+        self.block = block
+    def __enter__(self):
+        self.block._lock_attribute_creation = True
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.block._lock_attribute_creation = False
+
+
 class PhysicalParameterBlock(ProcessBlockData,
                              property_meta.HasPropertyClassMetadata):
     """
@@ -83,6 +95,69 @@ class PhysicalParameterBlock(ProcessBlockData,
         if not hasattr(self, "_state_block_class"):
             self._state_block_class = None
 
+        # By default, property packages do not include inherent reactions
+        self._has_inherent_reactions = False
+
+        # This is a dict to store default property scaling factors. They are
+        # defined in the parameter block to provide a universal default for
+        # quantities in a particular kind of state block.  For example, you can
+        # set flow scaling once instead of for every state block. Some of these
+        # may be left for the user to set and some may be defined in a property
+        # module where reasonable defaults can be defined a priori. See
+        # set_default_scaling, get_default_scaling, and unset_default_scaling
+        self.default_scaling_factor = {}
+
+    def set_default_scaling(self, attrbute, value, index=None):
+        """Set a default scaling factor for a property.
+
+        Args:
+            attribute: property attribute name
+            value: default scaling factor
+            index: for indexed properties, if this is not provied the scaling
+                factor default applies to all indexed elements where specific
+                indexes are no specifcally specified.
+
+        Returns:
+            None
+        """
+        self.default_scaling_factor[(attrbute, index)] = value
+
+    def unset_default_scaling(self, attrbute, index=None):
+        """Remove a previously set default value
+
+        Args:
+            attribute: property attribute name
+            index: optional index for indexed properties
+
+        Returns:
+            None
+        """
+        try:
+            del self.default_scaling_factor[(attrbute, index)]
+        except KeyError:
+            pass
+
+    def get_default_scaling(self, attrbute, index=None):
+        """ Returns a default scale factor for a property
+
+        Args:
+            attribute: property attribute name
+            index: optional index for indexed properties
+
+        Returns:
+            None
+        """
+        try:
+            # If a specific component data index exists
+            return self.default_scaling_factor[(attrbute, index)]
+        except KeyError:
+            try:
+                # indexed, but no specifc index?
+                return self.default_scaling_factor[(attrbute, None)]
+            except KeyError:
+                # Can't find a default scale factor for what you asked for
+                return None
+
     @property
     def state_block_class(self):
         if self._state_block_class is not None:
@@ -100,6 +175,10 @@ class PhysicalParameterBlock(ProcessBlockData,
                      "_state_block_class attribute instead.")
         self._state_block_class = val
 
+    @property
+    def has_inherent_reactions(self):
+        return self._has_inherent_reactions
+
     def build_state_block(self, *args, **kwargs):
         """
         Methods to construct a StateBlock assoicated with this
@@ -119,10 +198,12 @@ class PhysicalParameterBlock(ProcessBlockData,
             for i in initialize.keys():
                 initialize[i]["parameters"] = self
 
-        return self.state_block_class(*args,
-                                      **kwargs,
-                                      default=default,
-                                      initialize=initialize)
+        return self.state_block_class(  # pylint: disable=not-callable
+            *args,
+            **kwargs,
+            default=default,
+            initialize=initialize
+        )
 
     def get_phase_component_set(self):
         """
@@ -240,6 +321,9 @@ class PhysicalParameterBlock(ProcessBlockData,
             raise PropertyPackageError("Property package {} has not defined a "
                                        "phase list.".format(self.name))
 
+        # Also check that the phase-component set has been created.
+        self.get_phase_component_set()
+
     def _make_component_objects(self):
         _log.warning("DEPRECATED: {} appears to be an old-style property "
                      "package. It will be automatically converted to a "
@@ -288,6 +372,70 @@ class StateBlock(ProcessBlock):
         multiple StateBlockData objects simultaneously.
     """
 
+    @property
+    def component_list(self):
+        return self._return_component_list()
+
+    def _return_component_list(self):
+        return self._get_parameter_block().component_list
+
+    @property
+    def phase_list(self):
+        return self._return_phase_list()
+
+    def _return_phase_list(self):
+        return self._get_parameter_block().phase_list
+
+    @property
+    def phase_component_set(self):
+        return self._return_phase_component_set()
+
+    def _return_phase_component_set(self):
+        return self._get_parameter_block().get_phase_component_set()
+
+    @property
+    def has_inherent_reactions(self):
+        return self._has_inherent_reactions()
+
+    def _has_inherent_reactions(self):
+        return self._get_parameter_block().has_inherent_reactions
+
+    # Need to separate the existence of inherent reactions from whether they
+    # should be included in material balances
+    # For some cases, Using an apparent species basis means they can be ignored
+    @property
+    def include_inherent_reactions(self):
+        return self._include_inherent_reactions()
+
+    def _include_inherent_reactions(self):
+        return self._get_parameter_block().has_inherent_reactions
+
+    def _get_parameter_block(self):
+        # PYLINT-WHY: self._block_data_config_default is set elsewhere,
+        # and is supposed to already exist as an attribute by the time
+        # (or, it will raise AttributeError at L410)
+        # this method is called, so the pylint errors here are false positives
+        # pylint: disable=no-member,access-member-before-definition
+        try:
+            return self._block_data_config_default["parameters"]
+        except (KeyError, TypeError):
+            # Need to get parameters from initialize dict
+            # We will also confirm these are all the same whilst we are at it
+            param = None
+            if self._block_data_config_default is None:
+                self._block_data_config_default = {}
+            for v in self._block_data_config_initialize.values():
+                if param is None:
+                    param = v["parameters"]
+                elif param is not v["parameters"]:
+                    raise ConfigurationError(
+                        "{} StateBlock must use the same parameter block for "
+                        "elements. When using the initialize argument, please "
+                        "ensure that the same value is used for all parameter "
+                        "keys.".format(self.name))
+            self._block_data_config_default["parameters"] = param
+        return param
+
     def initialize(self, *args, **kwargs):
         """
         This is a default initialization routine for StateBlocks to ensure
@@ -434,6 +582,50 @@ should be constructed in this state block,
 **True** - StateBlock should calculate phase equilibrium,
 **False** - StateBlock should not calculate phase equilibrium.}"""))
 
+    def __init__(self, *args, **kwargs):
+        self._lock_attribute_creation = False
+        super().__init__(*args, **kwargs)
+
+    def lock_attribute_creation_context(self):
+        """Returns a context manager that does not allow attributes to be created
+        while in the context and allows attributes to be created normally outside
+        the context.
+        """
+        return _lock_attribute_creation_context(self)
+
+    def is_property_constructed(self, attr):
+        """Returns True if the attribute ``attr`` already exists, or false if it
+        would be added in ``__getattr__``, or does not exist.
+
+        Args:
+            attr (str): Attribute name to check
+
+        Return:
+            True if the attribute is already constructed, False otherwise
+        """
+        with self.lock_attribute_creation_context():
+            return hasattr(self, attr)
+
+    @property
+    def component_list(self):
+        return self.parent_component()._return_component_list()
+
+    @property
+    def phase_list(self):
+        return self.parent_component()._return_phase_list()
+
+    @property
+    def phase_component_set(self):
+        return self.parent_component()._return_phase_component_set()
+
+    @property
+    def has_inherent_reactions(self):
+        return self.parent_component()._has_inherent_reactions()
+
+    @property
+    def include_inherent_reactions(self):
+        return self.parent_component()._include_inherent_reactions()
+
     def build(self):
         """
         General build method for StateBlockDatas.
@@ -608,6 +800,9 @@ should be constructed in this state block,
             attr: an attribute to create and return. Should be a property
                   component.
         """
+        if self._lock_attribute_creation:
+            raise AttributeError(
+                f"{attr} does not exist, and attribute creation is locked.")
 
         def clear_call_list(self, attr):
             """Local method for cleaning up call list when a call is handled.
@@ -704,7 +899,7 @@ should be constructed in this state block,
             raise PropertyNotSupportedError(
                     '{} {} is not supported by property package (property is '
                     'not listed in package metadata properties).'
-                    .format(self.name, attr, attr))
+                    .format(self.name, attr))
 
         # Get method name from resulting properties
         try:
@@ -780,3 +975,15 @@ should be constructed in this state block,
         comp = getattr(self, attr)
         clear_call_list(self, attr)
         return comp
+
+    def calculate_scaling_factors(self):
+        super().calculate_scaling_factors()
+        # Get scaling factor defaults, if no scaling factor set
+        for v in self.component_data_objects(
+                (Constraint, Var, Expression), descend_into=False):
+            if iscale.get_scaling_factor(v) is None:  # don't replace if set
+                name = v.getname().split("[")[0]
+                index = v.index()
+                sf = self.config.parameters.get_default_scaling(name, index)
+                if sf is not None:
+                    iscale.set_scaling_factor(v, sf)
