@@ -1,21 +1,22 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 This module contains classes for reaction blocks and reaction parameter blocks.
 """
 
 # Import Pyomo libraries
 from pyomo.common.config import ConfigBlock, ConfigValue, In
+from pyomo.environ import Var, Constraint, Expression
 
 # Import IDAES cores
 from idaes.core.process_block import ProcessBlock
@@ -23,12 +24,19 @@ from idaes.core import ProcessBlockData
 from idaes.core import property_meta
 from idaes.core import MaterialFlowBasis
 from idaes.core.util.exceptions import (BurntToast,
+                                        ConfigurationError,
                                         PropertyNotSupportedError,
                                         PropertyPackageError)
 from idaes.core.util.config import (is_physical_parameter_block,
                                     is_reaction_parameter_block,
                                     is_state_block)
 from idaes.core.util.misc import add_object_reference
+# WHY on Python 3.6, using the alternate syntax "import idaes.core.util.scaling as iscale"
+# fails with "AttributeError: module 'idaes' has no attribute 'core'"
+# this is likely due to a bug/limitation in how the Python import mechanism resolves circular imports
+# for more information, see https://stackoverflow.com/questions/24807434
+# and the official Python bug report: http://bugs.python.org/issue30024
+from idaes.core.util import scaling as iscale
 import idaes.logger as idaeslog
 
 _log = idaeslog.getLogger(__name__)
@@ -36,9 +44,20 @@ _log = idaeslog.getLogger(__name__)
 # Some more information about this module
 __author__ = "Andrew Lee, John Eslick"
 
-__all__ = ['ReactionBlockData',
-           'ReactionBlock',
-           'ReactionParameterBlock']
+__all__ = [
+    'ReactionBlock',  # pylint: disable=undefined-all-variable
+    'ReactionParameterBlock'  # pylint: disable=undefined-all-variable
+]
+
+
+class _lock_attribute_creation_context(object):
+    """Context manager to lock creation of new attributes on a state block"""
+    def __init__(self, block):
+        self.block = block
+    def __enter__(self):
+        self.block._lock_attribute_creation = True
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.block._lock_attribute_creation = False
 
 
 class ReactionParameterBlock(ProcessBlockData,
@@ -82,6 +101,66 @@ class ReactionParameterBlock(ProcessBlockData,
         self._validate_property_parameter_units()
         self._validate_property_parameter_properties()
 
+        # This is a dict to store default property scaling factors. They are
+        # defined in the parameter block to provide a universal default for
+        # quantities in a particular kind of state block.  For example, you can
+        # set flow scaling once instead of for every state block. Some of these
+        # may be left for the user to set and some may be defined in a property
+        # module where reasonable defaults can be defined a priori. See
+        # set_default_scaling, get_default_scaling, and unset_default_scaling
+        self.default_scaling_factor = {}
+
+    def set_default_scaling(self, attrbute, value, index=None):
+        """Set a default scaling factor for a property.
+
+        Args:
+            attribute: property attribute name
+            value: default scaling factor
+            index: for indexed properties, if this is not provied the scaling
+                factor default applies to all indexed elements where specific
+                indexes are no specifcally specified.
+
+        Returns:
+            None
+        """
+        self.default_scaling_factor[(attrbute, index)] = value
+
+    def unset_default_scaling(self, attrbute, index=None):
+        """Remove a previously set default value
+
+        Args:
+            attribute: property attribute name
+            index: optional index for indexed properties
+
+        Returns:
+            None
+        """
+        try:
+            del self.default_scaling_factor[(attrbute, index)]
+        except KeyError:
+            pass
+
+    def get_default_scaling(self, attrbute, index=None):
+        """ Returns a default scale factor for a property
+
+        Args:
+            attribute: property attribute name
+            index: optional index for indexed properties
+
+        Returns:
+            None
+        """
+        try:
+            # If a specific component data index exists
+            return self.default_scaling_factor[(attrbute, index)]
+        except KeyError:
+            try:
+                # indexed, but no specifc index?
+                return self.default_scaling_factor[(attrbute, None)]
+            except KeyError:
+                # Can't find a default scale factor for what you asked for
+                return None
+
     @property
     def reaction_block_class(self):
         if self._reaction_block_class is not None:
@@ -118,10 +197,12 @@ class ReactionParameterBlock(ProcessBlockData,
             for i in initialize.keys():
                 initialize[i]["parameters"] = self
 
-        return self.reaction_block_class(*args,
-                                         **kwargs,
-                                         default=default,
-                                         initialize=initialize)
+        return self.reaction_block_class(  # pylint: disable=not-callable
+            *args,
+            **kwargs,
+            default=default,
+            initialize=initialize
+        )
 
     def _validate_property_parameter_units(self):
         """
@@ -193,6 +274,7 @@ class ReactionBlockBase(ProcessBlock):
         PropertyData objects, and contains methods that can be applied to
         multiple ReactionBlockData objects simultaneously.
     """
+
     def initialize(self, *args):
         """
         This is a default initialization routine for ReactionBlocks to ensure
@@ -244,6 +326,42 @@ should be constructed in this reaction block,
 **True** - ReactionBlock should enforce equilibrium constraints,
 **False** - ReactionBlock should not enforce equilibrium constraints.}"""))
 
+    def __init__(self, *args, **kwargs):
+        self._lock_attribute_creation = False
+        super().__init__(*args, **kwargs)
+
+    @property
+    def component_list(self):
+        return self.state_ref.component_list
+
+    @property
+    def phase_list(self):
+        return self.state_ref.phase_list
+
+    @property
+    def phase_component_set(self):
+        return self.state_ref.phase_component_set
+
+    def lock_attribute_creation_context(self):
+        """Returns a context manager that does not allow attributes to be created
+        while in the context and allows attributes to be created normally outside
+        the context.
+        """
+        return _lock_attribute_creation_context(self)
+
+    def is_property_constructed(self, attr):
+        """Returns True if the attribute ``attr`` already exists, or false if it
+        would be added in ``__getattr__``, or does not exist.
+
+        Args:
+            attr (str): Attribute name to check
+
+        Return:
+            True if the attribute is already constructed, False otherwise 
+        """
+        with self.lock_attribute_creation_context():
+            return hasattr(self, attr)
+
     def build(self):
         """
         General build method for PropertyBlockDatas. Inheriting models should
@@ -318,6 +436,9 @@ should be constructed in this reaction block,
             attr: an attribute to create and return. Should be a property
                   component.
         """
+        if self._lock_attribute_creation:
+            raise AttributeError(
+                f"{attr} does not exist, and attribute creation is locked.")
 
         def clear_call_list(self, attr):
             """Local method for cleaning up call list when a call is handled.
@@ -414,7 +535,7 @@ should be constructed in this reaction block,
             raise PropertyNotSupportedError(
                     '{} {} is not supported by reaction package (property is '
                     'not listed in get_supported_properties).'
-                    .format(self.name, attr, attr))
+                    .format(self.name, attr))
 
         # Get method name from get_supported_properties
         try:
@@ -490,3 +611,15 @@ should be constructed in this reaction block,
         comp = getattr(self, attr)
         clear_call_list(self, attr)
         return comp
+
+    def calculate_scaling_factors(self):
+        super().calculate_scaling_factors()
+        # Get scaling factor defaults, if no scaling factor set
+        for v in self.component_data_objects(
+            (Constraint, Var, Expression), descend_into=False):
+            if iscale.get_scaling_factor(v) is None:  # don't replace if set
+                name = v.getname().split("[")[0]
+                index = v.index()
+                sf = self.config.parameters.get_default_scaling(name, index)
+                if sf is not None:
+                    iscale.set_scaling_factor(v, sf)
\ No newline at end of file
