@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 Tests for ControlVolumeBlockData.
 
@@ -32,6 +32,7 @@ from idaes.core.util.exceptions import (BalanceTypeNotSupportedError,
                                         PropertyNotSupportedError)
 from idaes.core.util.testing import (PhysicalParameterTestBlock,
                                      ReactionParameterTestBlock)
+import idaes.logger as idaeslog
 
 
 # -----------------------------------------------------------------------------
@@ -71,6 +72,7 @@ def test_base_build():
 
     assert hasattr(m.fs.config, "time")
 
+
 # -----------------------------------------------------------------------------
 # Test add_geometry
 @pytest.mark.unit
@@ -273,6 +275,7 @@ def test_add_phase_fractions():
     m.fs.cv = ControlVolume0DBlock(default={"property_package": m.fs.pp,
                                             "reaction_package": m.fs.rp})
 
+    m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv._add_phase_fractions()
 
     assert isinstance(m.fs.cv.phase_fraction, Var)
@@ -293,6 +296,7 @@ def test_add_phase_fractions_single_phase():
     m.fs.cv = ControlVolume0DBlock(default={"property_package": m.fs.pp,
                                             "reaction_package": m.fs.rp})
 
+    m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv._add_phase_fractions()
 
     assert isinstance(m.fs.cv.phase_fraction, Expression)
@@ -465,10 +469,10 @@ def test_rxn_rate_conv_mole_mass():
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
     for t in m.fs.time:
-        m.fs.cv.properties_out[t].mw = {"c1": 2, "c2": 3}
+        m.fs.cv.properties_out[t].mw_comp = {"c1": 2, "c2": 3}
         for j in m.fs.pp.component_list:
             assert (m.fs.cv._rxn_rate_conv(t, j, has_rate_reactions=True) ==
-                    1/m.fs.cv.properties_out[t].mw[j])
+                    1/m.fs.cv.properties_out[t].mw_comp[j])
 
 
 @pytest.mark.unit
@@ -488,10 +492,10 @@ def test_rxn_rate_conv_mass_mole():
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
     for t in m.fs.time:
-        m.fs.cv.properties_out[t].mw = {"c1": 2, "c2": 3}
+        m.fs.cv.properties_out[t].mw_comp = {"c1": 2, "c2": 3}
         for j in m.fs.pp.component_list:
             assert (m.fs.cv._rxn_rate_conv(t, j, has_rate_reactions=True) ==
-                    m.fs.cv.properties_out[t].mw[j])
+                    m.fs.cv.properties_out[t].mw_comp[j])
 
 
 # -----------------------------------------------------------------------------
@@ -535,6 +539,7 @@ def test_add_material_balances_default():
 
     assert_units_consistent(m)
 
+
 # -----------------------------------------------------------------------------
 # Test add_phase_component_balances
 @pytest.mark.unit
@@ -726,6 +731,53 @@ def test_add_phase_component_balances_eq_rxns_no_idx():
         m.fs.cv.add_phase_component_balances(has_equilibrium_reactions=True)
 
 
+@pytest.mark.unit
+def test_add_phase_component_balances_in_rxns():
+    m = ConcreteModel()
+    m.fs = Flowsheet(default={"dynamic": False})
+    m.fs.pp = PhysicalParameterTestBlock()
+
+    # Set property package to contain inherent reactions
+    m.fs.pp._has_inherent_reactions = True
+
+    m.fs.cv = ControlVolume0DBlock(default={"property_package": m.fs.pp})
+
+    m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
+
+    mb = m.fs.cv.add_phase_component_balances()
+
+    assert isinstance(mb, Constraint)
+    assert len(mb) == 4
+    assert isinstance(m.fs.cv.inherent_reaction_generation, Var)
+    assert isinstance(m.fs.cv.inherent_reaction_extent, Var)
+    assert isinstance(m.fs.cv.inherent_reaction_stoichiometry_constraint,
+                      Constraint)
+
+    assert_units_consistent(m)
+
+
+@pytest.mark.unit
+def test_add_phase_component_balances_in_rxns_no_idx():
+    m = ConcreteModel()
+    m.fs = Flowsheet(default={"dynamic": False})
+    m.fs.pp = PhysicalParameterTestBlock()
+
+    # Set property package to contain inherent reactions
+    m.fs.pp._has_inherent_reactions = True
+    # delete inherent_Reaction_dix to trigger exception
+    m.fs.pp.del_component(m.fs.pp.inherent_reaction_idx)
+
+    m.fs.cv = ControlVolume0DBlock(default={"property_package": m.fs.pp})
+
+    m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
+
+    with pytest.raises(PropertyNotSupportedError,
+                       match="fs.cv Property package does not contain a "
+                       "list of inherent reactions \(inherent_reaction_idx\), "
+                       "but include_inherent_reactions is True."):
+        m.fs.cv.add_phase_component_balances()
+
+
 @pytest.mark.unit
 def test_add_phase_component_balances_eq_rxns_no_ReactionBlock():
     m = ConcreteModel()
@@ -840,7 +892,7 @@ def test_add_phase_component_balances_custom_molar_term():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.phase_list,
                            m.fs.cv.config.property_package.component_list)
 
@@ -870,7 +922,7 @@ def test_add_phase_component_balances_custom_molar_term_no_mw():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.phase_list,
                            m.fs.cv.config.property_package.component_list)
 
@@ -896,7 +948,7 @@ def test_add_phase_component_balances_custom_molar_term_mass_flow_basis():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.phase_list,
                            m.fs.cv.config.property_package.component_list)
 
@@ -904,7 +956,7 @@ def test_add_phase_component_balances_custom_molar_term_mass_flow_basis():
         return m.fs.cv.test_var[t, p, j]*units.mol/units.s
 
     for t in m.fs.time:
-        m.fs.cv.properties_out[t].mw = Var(
+        m.fs.cv.properties_out[t].mw_comp = Var(
                 m.fs.cv.properties_out[t].config.parameters.component_list,
                 units=units.kg/units.mol)
 
@@ -931,7 +983,7 @@ def test_add_phase_component_balances_custom_molar_term_undefined_basis():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.phase_list,
                            m.fs.cv.config.property_package.component_list)
 
@@ -957,7 +1009,7 @@ def test_add_phase_component_balances_custom_mass_term():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.phase_list,
                            m.fs.cv.config.property_package.component_list)
 
@@ -987,7 +1039,7 @@ def test_add_phase_component_balances_custom_mass_term_no_mw():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.phase_list,
                            m.fs.cv.config.property_package.component_list)
 
@@ -1013,7 +1065,7 @@ def test_add_phase_component_balances_custom_mass_term_mole_flow_basis():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.phase_list,
                            m.fs.cv.config.property_package.component_list)
 
@@ -1021,7 +1073,7 @@ def test_add_phase_component_balances_custom_mass_term_mole_flow_basis():
         return m.fs.cv.test_var[t, p, j]*units.kg/units.s
 
     for t in m.fs.time:
-        m.fs.cv.properties_out[t].mw = Var(
+        m.fs.cv.properties_out[t].mw_comp = Var(
                 m.fs.cv.properties_out[t].config.parameters.component_list,
                 units=units.kg/units.mol)
 
@@ -1048,7 +1100,7 @@ def test_add_phase_component_balances_custom_mass_term_undefined_basis():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.phase_list,
                            m.fs.cv.config.property_package.component_list)
 
@@ -1152,6 +1204,7 @@ def test_add_total_component_balances_rate_rxns():
 
     assert_units_consistent(m)
 
+
 @pytest.mark.unit
 def test_add_total_component_balances_rate_rxns_no_ReactionBlock():
     m = ConcreteModel()
@@ -1264,6 +1317,53 @@ def test_add_total_component_balances_eq_rxns_no_ReactionBlock():
         m.fs.cv.add_total_component_balances(has_equilibrium_reactions=True)
 
 
+@pytest.mark.unit
+def test_add_total_component_balances_in_rxns():
+    m = ConcreteModel()
+    m.fs = Flowsheet(default={"dynamic": False})
+    m.fs.pp = PhysicalParameterTestBlock()
+
+    # Set property package to contain inherent reactions
+    m.fs.pp._has_inherent_reactions = True
+
+    m.fs.cv = ControlVolume0DBlock(default={"property_package": m.fs.pp})
+
+    m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
+
+    mb = m.fs.cv.add_total_component_balances()
+
+    assert isinstance(mb, Constraint)
+    assert len(mb) == 2
+    assert isinstance(m.fs.cv.inherent_reaction_generation, Var)
+    assert isinstance(m.fs.cv.inherent_reaction_extent, Var)
+    assert isinstance(m.fs.cv.inherent_reaction_stoichiometry_constraint,
+                      Constraint)
+
+    assert_units_consistent(m)
+
+
+@pytest.mark.unit
+def test_add_total_component_balances_in_rxns_no_idx():
+    m = ConcreteModel()
+    m.fs = Flowsheet(default={"dynamic": False})
+    m.fs.pp = PhysicalParameterTestBlock()
+
+    # Set property package to contain inherent reactions
+    m.fs.pp._has_inherent_reactions = True
+    # delete inherent_Reaction_dix to trigger exception
+    m.fs.pp.del_component(m.fs.pp.inherent_reaction_idx)
+
+    m.fs.cv = ControlVolume0DBlock(default={"property_package": m.fs.pp})
+
+    m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
+
+    with pytest.raises(PropertyNotSupportedError,
+                       match="fs.cv Property package does not contain a "
+                       "list of inherent reactions \(inherent_reaction_idx\), "
+                       "but include_inherent_reactions is True."):
+        m.fs.cv.add_total_component_balances()
+
+
 @pytest.mark.unit
 def test_add_total_component_balances_phase_eq_not_active():
     m = ConcreteModel()
@@ -1319,7 +1419,7 @@ def test_add_total_component_balances_custom_molar_term():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.component_list)
 
     def custom_method(t, j):
@@ -1348,7 +1448,7 @@ def test_add_total_component_balances_custom_molar_term_no_mw():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.component_list)
 
     def custom_method(t, j):
@@ -1373,14 +1473,14 @@ def test_add_total_component_balances_custom_molar_term_mass_flow_basis():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.component_list)
 
     def custom_method(t, j):
         return m.fs.cv.test_var[t, j]*units.mol/units.s
 
     for t in m.fs.time:
-        m.fs.cv.properties_out[t].mw = Var(
+        m.fs.cv.properties_out[t].mw_comp = Var(
                 m.fs.cv.properties_out[t].config.parameters.component_list,
                 units=units.kg/units.mol)
 
@@ -1407,7 +1507,7 @@ def test_add_total_component_balances_custom_molar_term_undefined_basis():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.component_list)
 
     def custom_method(t, j):
@@ -1432,7 +1532,7 @@ def test_add_total_component_balances_custom_mass_term():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.component_list)
 
     def custom_method(t, j):
@@ -1461,7 +1561,7 @@ def test_add_total_component_balances_custom_mass_term_no_mw():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.component_list)
 
     def custom_method(t, j):
@@ -1486,14 +1586,14 @@ def test_add_total_component_balances_custom_mass_term_mole_flow_basis():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.component_list)
 
     def custom_method(t, j):
         return m.fs.cv.test_var[t, j]*units.kg/units.s
 
     for t in m.fs.time:
-        m.fs.cv.properties_out[t].mw = Var(
+        m.fs.cv.properties_out[t].mw_comp = Var(
                 m.fs.cv.properties_out[t].config.parameters.component_list,
                 units=units.kg/units.mol)
 
@@ -1520,7 +1620,7 @@ def test_add_total_component_balances_custom_mass_term_undefined_basis():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.cv.config.property_package.component_list)
 
     def custom_method(t, j):
@@ -1649,7 +1749,7 @@ def test_add_total_element_balances_eq_rxns():
     m.fs.cv.add_reaction_blocks(has_equilibrium=True)
 
     with pytest.raises(ConfigurationError):
-        mb = m.fs.cv.add_total_element_balances(has_equilibrium_reactions=True)
+        m.fs.cv.add_total_element_balances(has_equilibrium_reactions=True)
 
 
 @pytest.mark.unit
@@ -1707,7 +1807,7 @@ def test_add_total_element_balances_custom_term():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time,
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time,
                            m.fs.pp.element_list)
 
     def custom_method(t, e):
@@ -1722,6 +1822,39 @@ def test_add_total_element_balances_custom_term():
     assert_units_consistent(m)
 
 
+@pytest.mark.unit
+def test_add_total_element_balances_lineraly_dependent(caplog):
+    m = ConcreteModel()
+    m.fs = Flowsheet(default={"dynamic": False})
+    m.fs.pp = PhysicalParameterTestBlock()
+
+    # Change elemental composition to introduce dependency
+    m.fs.pp.element_comp = {"c1": {"H": 0, "He": 0, "Li": 1},
+                            "c2": {"H": 1, "He": 2, "Li": 0}}
+
+    m.fs.cv = ControlVolume0DBlock(default={"property_package": m.fs.pp})
+
+    m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
+
+    mb = m.fs.cv.add_total_element_balances()
+    # Check that logger message was recorded and has the right level
+    msg = ("fs.cv detected linearly dependent element balance equations. "
+           "Element balances will NOT be written for the following elements: "
+           "['He']")
+    assert msg in caplog.text
+    for record in caplog.records:
+        assert record.levelno == idaeslog.INFO_LOW
+
+    assert isinstance(mb, Constraint)
+    assert len(mb) == 2
+    for e in mb:
+        # H and Li are not lineraly dependent and should have constraints
+        assert e in [(0, "H"), (0, "Li")]
+        # He is lineraly dependent on H and should be skipped
+
+    assert_units_consistent(m)
+
+
 # -----------------------------------------------------------------------------
 # Test unsupported material balance types
 @pytest.mark.unit
@@ -1936,7 +2069,7 @@ def test_add_total_enthalpy_balances_custom_term():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time)
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time)
 
     def custom_method(t):
         return m.fs.cv.test_var[t]*units.J/units.s
@@ -1982,7 +2115,7 @@ def test_add_total_enthalpy_balances_dh_rxn_rate_rxns():
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
     m.fs.cv.add_phase_component_balances(has_rate_reactions=True)
 
-    eb = m.fs.cv.add_total_enthalpy_balances(has_heat_of_reaction=True)
+    m.fs.cv.add_total_enthalpy_balances(has_heat_of_reaction=True)
     assert isinstance(m.fs.cv.heat_of_reaction, Expression)
 
     assert_units_consistent(m)
@@ -2003,7 +2136,7 @@ def test_add_total_enthalpy_balances_dh_rxn_equil_rxns():
     m.fs.cv.add_reaction_blocks(has_equilibrium=True)
     m.fs.cv.add_phase_component_balances(has_equilibrium_reactions=True)
 
-    eb = m.fs.cv.add_total_enthalpy_balances(has_heat_of_reaction=True)
+    m.fs.cv.add_total_enthalpy_balances(has_heat_of_reaction=True)
     assert isinstance(m.fs.cv.heat_of_reaction, Expression)
 
     assert_units_consistent(m)
@@ -2128,7 +2261,7 @@ def test_add_total_pressure_balances_custom_term():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=False)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().config.time)
+    m.fs.cv.test_var = Var(m.fs.cv.flowsheet().time)
 
     def custom_method(t):
         return m.fs.cv.test_var[t]*units.Pa
@@ -2240,7 +2373,7 @@ def test_initialize():
     m.fs.cv.add_state_blocks(has_phase_equilibrium=True)
     m.fs.cv.add_reaction_blocks(has_equilibrium=False)
 
-    f= m.fs.cv.initialize(state_args={})
+    f = m.fs.cv.initialize()
 
     for t in m.fs.time:
         assert m.fs.cv.properties_in[t].init_test is True
@@ -2348,9 +2481,7 @@ def test_get_performance_contents():
     for k in dd["exprs"].keys():
         assert k in ["Heat of Reaction Term"]
 
-    assert len(dd["params"]) == 2
-    for k in dd["params"].keys():
-        assert k in ["Pressure Scaling", "Energy Scaling"]
+    assert len(dd["params"]) == 0
 
 
 @pytest.mark.unit
@@ -2412,9 +2543,7 @@ def test_get_performance_contents_elemental():
                      "Element Flow Out [p2, He]",
                      "Element Flow Out [p2, Li]"]
 
-    assert len(dd["params"]) == 2
-    for k in dd["params"].keys():
-        assert k in ["Pressure Scaling", "Energy Scaling"]
+    assert len(dd["params"]) == 0
 
 
 @pytest.mark.unit
