@@ -1,20 +1,20 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 Base class for unit models
 """
 
-from pyomo.environ import Reference, SolverFactory
+from pyomo.environ import Reference
 from pyomo.network import Port
 from pyomo.common.config import ConfigValue, In
 
@@ -30,7 +30,9 @@ from idaes.core.util.exceptions import (BurntToast,
                                         PropertyPackageError,
                                         BalanceTypeNotSupportedError)
 from idaes.core.util.tables import create_stream_table_dataframe
+import idaes.core.util.unit_costing
 import idaes.logger as idaeslog
+from idaes.core.util import get_solver
 
 __author__ = "John Eslick, Qi Chen, Andrew Lee"
 
@@ -144,7 +146,7 @@ Must be True if dynamic = True,
 
         # Get dict of Port members and names
         member_list = block[
-                blk.flowsheet().config.time.first()].define_port_members()
+                blk.flowsheet().time.first()].define_port_members()
 
         # Create References for port members
         for s in member_list:
@@ -154,7 +156,7 @@ Must be True if dynamic = True,
                 slicer = block[:].component(member_list[s].local_name)[...]
 
             r = Reference(slicer)
-            setattr(blk, "_"+name+"_"+s+"_ref", r)
+            setattr(blk, "_"+s+"_"+name+"_ref", r)
 
             # Add Reference to Port
             p.add(r, s)
@@ -220,13 +222,13 @@ Must be True if dynamic = True,
         if isinstance(block, ControlVolumeBlockData):
             try:
                 member_list = (block.properties_in[
-                                    block.flowsheet().config.time.first()]
+                                    block.flowsheet().time.first()]
                                .define_port_members())
                 p._state_block = (block.properties_in, )
             except AttributeError:
                 try:
                     member_list = (block.properties[
-                                    block.flowsheet().config.time.first(), 0]
+                                    block.flowsheet().time.first(), 0]
                                    .define_port_members())
                     if block._flow_direction == FlowDirection.forward:
                         p._state_block = (block.properties,
@@ -242,7 +244,7 @@ Must be True if dynamic = True,
                             "package.".format(blk.name))
         elif isinstance(block, StateBlock):
             member_list = block[
-                    blk.flowsheet().config.time.first()].define_port_members()
+                    blk.flowsheet().time.first()].define_port_members()
             p._state_block = (block, )
         else:
             raise ConfigurationError(
@@ -309,7 +311,7 @@ Must be True if dynamic = True,
                             .format(blk.name))
 
             r = Reference(slicer)
-            setattr(blk, "_"+name+"_"+s+"_ref", r)
+            setattr(blk, "_"+s+"_"+name+"_ref", r)
 
             # Add Reference to Port
             p.add(r, s)
@@ -375,13 +377,13 @@ Must be True if dynamic = True,
         if isinstance(block, ControlVolumeBlockData):
             try:
                 member_list = (block.properties_out[
-                                    block.flowsheet().config.time.first()]
+                                    block.flowsheet().time.first()]
                                .define_port_members())
                 p._state_block = (block.properties_out, )
             except AttributeError:
                 try:
                     member_list = (block.properties[
-                                    block.flowsheet().config.time.first(), 0]
+                                    block.flowsheet().time.first(), 0]
                                    .define_port_members())
                     if block._flow_direction == FlowDirection.forward:
                         p._state_block = (block.properties,
@@ -397,7 +399,7 @@ Must be True if dynamic = True,
                             "package.".format(blk.name))
         elif isinstance(block, StateBlock):
             member_list = block[
-                    blk.flowsheet().config.time.first()].define_port_members()
+                    blk.flowsheet().time.first()].define_port_members()
             p._state_block = (block, )
         else:
             raise ConfigurationError(
@@ -465,7 +467,7 @@ Must be True if dynamic = True,
                             .format(blk.name))
 
             r = Reference(slicer)
-            setattr(blk, "_"+name+"_"+s+"_ref", r)
+            setattr(blk, "_"+s+"_"+name+"_ref", r)
 
             # Add Reference to Port
             p.add(r, s)
@@ -509,7 +511,7 @@ Must be True if dynamic = True,
                     "once per UnitModel.".format(self.name))
 
         # Get a representative time point for testing
-        rep_time = self.flowsheet().config.time.first()
+        rep_time = self.flowsheet().time.first()
         if state_1[rep_time].params is not state_2[rep_time].params:
             raise ConfigurationError(
                     "{} add_state_material_balances method was provided with "
@@ -523,17 +525,17 @@ Must be True if dynamic = True,
                 state_1[rep_time].default_material_balance_type()
             )
 
-        phase_list = state_1[rep_time].params.phase_list
-        component_list = state_1[rep_time].params.component_list
+        phase_list = state_1.phase_list
+        component_list = state_1.component_list
+        pc_set = state_1.phase_component_set
 
         if balance_type == MaterialBalanceType.componentPhase:
             # TODO : Should we include an optional phase equilibrium term here
             # to allow for systems where a phase-transition may occur?
 
             @self.Constraint(
-                self.flowsheet().config.time,
-                phase_list,
-                component_list,
+                self.flowsheet().time,
+                pc_set,
                 doc="State material balances",
             )
             def state_material_balances(b, t, p, j):
@@ -544,38 +546,30 @@ Must be True if dynamic = True,
         elif balance_type == MaterialBalanceType.componentTotal:
 
             @self.Constraint(
-                self.flowsheet().config.time,
+                self.flowsheet().time,
                 component_list,
                 doc="State material balances",
             )
             def state_material_balances(b, t, j):
                 return sum(
                     state_1[t].get_material_flow_terms(p, j)
-                    for p in phase_list
+                    for p in phase_list if (p, j) in pc_set
                 ) == sum(
                     state_2[t].get_material_flow_terms(p, j)
-                    for p in phase_list
+                    for p in phase_list if (p, j) in pc_set
                 )
 
         elif balance_type == MaterialBalanceType.total:
 
             @self.Constraint(
-                self.flowsheet().config.time,
+                self.flowsheet().time,
                 doc="State material balances",
             )
             def state_material_balances(b, t):
                 return sum(
-                    sum(
-                        state_1[t].get_material_flow_terms(p, j)
-                        for j in component_list
-                    )
-                    for p in phase_list
+                    state_1[t].get_material_flow_terms(p, j) for p, j in pc_set
                 ) == sum(
-                    sum(
-                        state_2[t].get_material_flow_terms(p, j)
-                        for j in component_list
-                    )
-                    for p in phase_list
+                    state_2[t].get_material_flow_terms(p, j) for p, j in pc_set
                 )
 
         elif balance_type == MaterialBalanceType.elementTotal:
@@ -612,7 +606,7 @@ Must be True if dynamic = True,
                     f"developer to develop a unit specific stream table.")
 
     def initialize(blk, state_args=None, outlvl=idaeslog.NOTSET,
-                   solver='ipopt', optarg={'tol': 1e-6}):
+                   solver=None, optarg=None):
         '''
         This is a general purpose initialization routine for simple unit
         models. This method assumes a single ControlVolume block called
@@ -628,19 +622,22 @@ Must be True if dynamic = True,
                            initialization (see documentation of the specific
                            property package) (default = {}).
             outlvl : sets output level of initialization routine
-            optarg : solver options dictionary object (default={'tol': 1e-6})
+            optarg : solver options dictionary object (default=None, use
+                     default solver options)
             solver : str indicating which solver to use during
-                     initialization (default = 'ipopt')
+                     initialization (default = None, use default IDAES solver)
 
         Returns:
             None
         '''
+        if optarg is None:
+            optarg = {}
+
         # Set solver options
         init_log = idaeslog.getInitLogger(blk.name, outlvl, tag="unit")
         solve_log = idaeslog.getSolveLogger(blk.name, outlvl, tag="unit")
 
-        opt = SolverFactory(solver)
-        opt.options = optarg
+        opt = get_solver(solver, optarg)
 
         # ---------------------------------------------------------------------
         # Initialize control volume block
@@ -655,6 +652,11 @@ Must be True if dynamic = True,
 
         # ---------------------------------------------------------------------
         # Solve unit
+
+        # if costing block exists, deactivate
+        if hasattr(blk, "costing"):
+            blk.costing.deactivate()
+
         with idaeslog.solver_log(solve_log, idaeslog.DEBUG) as slc:
             results = opt.solve(blk, tee=slc.tee)
 
@@ -662,9 +664,13 @@ Must be True if dynamic = True,
             "Initialization Step 2 {}.".format(idaeslog.condition(results))
         )
 
+        # if costing block exists, activate and initialize
+        if hasattr(blk, "costing"):
+            blk.costing.activate()
+            idaes.core.util.unit_costing.initialize(blk.costing)
         # ---------------------------------------------------------------------
         # Release Inlet state
-        blk.control_volume.release_state(flags, outlvl+1)
+        blk.control_volume.release_state(flags, outlvl)
 
         init_log.info('Initialization Complete: {}'
                       .format(idaeslog.condition(results)))
