@@ -1,30 +1,30 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 Tests for math util methods.
 """
 
 import pytest
-from pyomo.environ import (Block, ConcreteModel, Constraint, Expression,
-                           Set, SolverFactory, Var, value, Param, Reals,
-                           TransformationFactory, TerminationCondition,
-                           exp)
+from pyomo.environ import (Block, ConcreteModel, Constraint, Expression, exp,
+                           Set, Var, value, Param, Reals,
+                           TransformationFactory, TerminationCondition)
 from pyomo.network import Arc, Port
+from pyomo.core.base.units_container import UnitsError
 
-from idaes.core import (FlowsheetBlock, 
-                        MaterialBalanceType, 
+from idaes.core import (FlowsheetBlock,
+                        MaterialBalanceType,
                         EnergyBalanceType,
-                        MomentumBalanceType, 
+                        MomentumBalanceType,
                         declare_process_block_class,
                         PhysicalParameterBlock,
                         StateBlock,
@@ -42,18 +42,13 @@ from idaes.core.util.initialization import (fix_state_vars,
                                             propagate_state,
                                             solve_indexed_blocks,
                                             initialize_by_time_element)
+from idaes.core.util import get_solver
 
 __author__ = "Andrew Lee"
 
 
-# See if ipopt is available and set up solver
-if SolverFactory('ipopt').available():
-    solver = SolverFactory('ipopt')
-    solver.options = {'tol': 1e-6,
-                      'mu_init': 1e-8,
-                      'bound_push': 1e-8}
-else:
-    solver = None
+# Set up solver
+solver = get_solver()
 
 
 @declare_process_block_class("AqueousEnzymeParameterBlock")
@@ -62,12 +57,13 @@ class ParameterData(PhysicalParameterBlock):
     Parameter block for the aqueous enzyme reaction in the biochemical CSTR
     used by Christofides and Daoutidis, 1996, presented by Heineken et al, 1967
     """
+
     def build(self):
         super(ParameterData, self).build()
 
         # all components are in the aqueous phase
         self.phase_list = Set(initialize=['aq'])
-        self.component_list = Set(initialize=['S', 'E', 'C', 'P','Solvent'])
+        self.component_list = Set(initialize=['S', 'E', 'C', 'P', 'Solvent'])
 
         self.state_block_class = AqueousEnzymeStateBlock
 
@@ -85,6 +81,7 @@ class _AqueousEnzymeStateBlock(StateBlock):
     def initialize(blk):
         pass
 
+
 @declare_process_block_class("AqueousEnzymeStateBlock",
                              block_class=_AqueousEnzymeStateBlock)
 class AqueousEnzymeStateBlockData(StateBlockData):
@@ -92,27 +89,29 @@ class AqueousEnzymeStateBlockData(StateBlockData):
         super(AqueousEnzymeStateBlockData, self).build()
 
         self.conc_mol = Var(self._params.component_list,
-                             domain=Reals,
-                             doc='Component molar concentration [kmol/m^3]')
-        
+                            domain=Reals,
+                            doc='Component molar concentration [kmol/m^3]')
+
         self.flow_mol_comp = Var(self._params.component_list,
-                                 domain=Reals, 
+                                 domain=Reals,
                                  doc='Molar component flow rate [kmol/min]')
 
-        self.flow_vol = Var(domain=Reals,
-                             doc='Volumetric flow rate out of reactor [m^3/min]')
+        self.flow_vol = Var(
+            domain=Reals, doc='Volumetric flow rate out of reactor [m^3/min]')
 
         self.temperature = Var(initialize=303, domain=Reals,
                                doc='Temperature within reactor [K]')
 
         if not self.config.defined_state:
             self.conc_mol['Solvent'].fix(1.)
+
         def flow_mol_comp_rule(b, j):
             return b.flow_mol_comp[j] == b.flow_vol*b.conc_mol[j]
-        
-        self.flow_mol_comp_eqn = Constraint(self._params.component_list,
-                rule=flow_mol_comp_rule,
-                doc='Outlet component molar flow rate equation')
+
+        self.flow_mol_comp_eqn = Constraint(
+            self._params.component_list,
+            rule=flow_mol_comp_rule,
+            doc='Outlet component molar flow rate equation')
 
     def get_material_density_terms(b, p, j):
         return b.conc_mol[j]
@@ -134,12 +133,14 @@ class AqueousEnzymeStateBlockData(StateBlockData):
                 'temperature': b.temperature,
                 'flow_vol': b.flow_vol}
 
+
 @declare_process_block_class('EnzymeReactionParameterBlock')
 class EnzymeReactionParameterData(ReactionParameterBlock):
     '''
     Enzyme reaction:
     S + E <-> C -> P + E
     '''
+
     def build(self):
         super(EnzymeReactionParameterData, self).build()
 
@@ -163,21 +164,23 @@ class EnzymeReactionParameterData(ReactionParameterBlock):
                                             ('R3', 'aq', 'Solvent'): 0}
 
         self.act_energy = Param(self.rate_reaction_idx,
-                initialize={'R1': 8.0e3,
-                            'R2': 9.0e3,
-                            'R3': 1.0e4},
-                doc='Activation energy [kcal/kmol]')
+                                initialize={'R1': 8.0e3,
+                                            'R2': 9.0e3,
+                                            'R3': 1.0e4},
+                                doc='Activation energy [kcal/kmol]')
 
-        self.gas_const = Param(initialize=1.987, 
-                doc='Gas constant R [kcal/kmol/K]')
+        self.gas_const = Param(initialize=1.987,
+                               doc='Gas constant R [kcal/kmol/K]')
 
-        self.temperature_ref = Param(initialize=300.0, doc='Reference temperature')
+        self.temperature_ref = Param(initialize=300.0,
+                                     doc='Reference temperature')
 
-        self.k_rxn = Param(self.rate_reaction_idx,
-                initialize={'R1': 3.36e6,
-                            'R2': 1.80e6,
-                            'R3': 5.79e7},
-                doc='Pre-exponential rate constant in Arrhenius expression')
+        self.k_rxn = Param(
+            self.rate_reaction_idx,
+            initialize={'R1': 3.36e6,
+                        'R2': 1.80e6,
+                        'R3': 5.79e7},
+            doc='Pre-exponential rate constant in Arrhenius expression')
 
     #    self.reaction_block_class = EnzymeReactionBlock
 
@@ -190,11 +193,13 @@ class EnzymeReactionParameterData(ReactionParameterBlock):
                                'energy': 'kcal',
                                'holdup': 'kmol'})
 
+
 class _EnzymeReactionBlock(ReactionBlockBase):
     def initialize(blk):
         # initialize for reaction rates for each data object
         pass
 
+
 @declare_process_block_class('EnzymeReactionBlock',
                              block_class=_EnzymeReactionBlock)
 class EnzymeReactionBlockData(ReactionBlockDataBase):
@@ -202,43 +207,43 @@ class EnzymeReactionBlockData(ReactionBlockDataBase):
         super(EnzymeReactionBlockData, self).build()
 
         self.reaction_coef = Var(self._params.rate_reaction_idx,
-                         domain=Reals, doc='Reaction rate coefficient')
+                                 domain=Reals, doc='Reaction rate coefficient')
 
         self.reaction_rate = Var(self._params.rate_reaction_idx,
-                                 domain=Reals, 
+                                 domain=Reals,
                                  doc='Reaction rate [kmol/m^3/min]')
 
         self.dh_rxn = Param(self._params.rate_reaction_idx,
-                domain=Reals, doc='Heat of reaction',
-                initialize={'R1': 1e3/900/0.231,
-                            'R2': 1e3/900/0.231,
-                            'R3': 5e3/900/0.231})
+                            domain=Reals, doc='Heat of reaction',
+                            initialize={'R1': 1e3/900/0.231,
+                                        'R2': 1e3/900/0.231,
+                                        'R3': 5e3/900/0.231})
 
         def reaction_rate_rule(b, r):
             if r == 'R1':
-                return (b.reaction_rate[r] == 
-                        b.reaction_coef[r]*
+                return (b.reaction_rate[r] ==
+                        b.reaction_coef[r] *
                         b.state_ref.conc_mol['S']*b.state_ref.conc_mol['E'])
             elif r == 'R2':
                 return (b.reaction_rate[r] ==
-                        b.reaction_coef[r]*
+                        b.reaction_coef[r] *
                         b.state_ref.conc_mol['C'])
             elif r == 'R3':
                 return (b.reaction_rate[r] ==
-                        b.reaction_coef[r]*
+                        b.reaction_coef[r] *
                         b.state_ref.conc_mol['C'])
 
         self.reaction_rate_eqn = Constraint(self._params.rate_reaction_idx,
-                rule=reaction_rate_rule)
+                                            rule=reaction_rate_rule)
 
         def arrhenius_rule(b, r):
-            return (b.reaction_coef[r] == b._params.k_rxn[r]*
-                    exp(-b._params.act_energy[r]/b._params.gas_const/
+            return (b.reaction_coef[r] == b._params.k_rxn[r] *
+                    exp(-b._params.act_energy[r]/b._params.gas_const /
                         b.state_ref.temperature))
 
         self.arrhenius_eqn = Constraint(self._params.rate_reaction_idx,
-                rule=arrhenius_rule)
-    
+                                        rule=arrhenius_rule)
+
     def get_reaction_rate_basis(b):
         return MaterialFlowBasis.molar
 
@@ -292,12 +297,12 @@ def test_fix_state_vars_None_value(model):
 
 @pytest.mark.unit
 def test_fix_state_vars_guesses(model):
-    # Note that flow_mol_phase_comp is labled as compoennt_flow
+    # Note that flow_mol_phase_comp is labled as component_flow
     # in define_state_vars
     state_args = {"component_flow_phase": {("p1", "c1"): 1,
-                                     ("p1", "c2"): 2,
-                                     ("p2", "c1"): 3,
-                                     ("p2", "c2"): 4},
+                                           ("p1", "c2"): 2,
+                                           ("p2", "c1"): 3,
+                                           ("p2", "c2"): 4},
                   "pressure": 2e5,
                   "temperature": 500}
 
@@ -330,9 +335,9 @@ def test_fix_state_vars_partial_guesses(model):
     # Note that flow_mol_phase_comp is labled as compoennt_flow
     # in define_state_vars
     state_args = {"component_flow_phase": {("p1", "c1"): 1,
-                                     ("p1", "c2"): 2,
-                                     ("p2", "c1"): 3,
-                                     ("p2", "c2"): 4}}
+                                           ("p1", "c2"): 2,
+                                           ("p2", "c1"): 3,
+                                           ("p2", "c2"): 4}}
 
     flags = fix_state_vars(model.fs.sb, state_args)
 
@@ -363,8 +368,8 @@ def test_fix_state_vars_guesses_mismatch_index(model):
     # Note that flow_mol_phase_comp is labled as compoennt_flow
     # in define_state_vars
     state_args = {"component_flow_phase": {("p1", "c1"): 1,
-                                     ("p1", "c2"): 2,
-                                     ("p2", "c1"): 3},
+                                           ("p1", "c2"): 2,
+                                           ("p2", "c1"): 3},
                   "pressure": 2e5,
                   "temperature": 500}
 
@@ -412,9 +417,9 @@ def test_fix_state_vars_fixed_guesses(model):
     model.fs.sb.pressure.fix(1.5e5)
 
     state_args = {"component_flow_phase": {("p1", "c1"): 1,
-                                     ("p1", "c2"): 2,
-                                     ("p2", "c1"): 3,
-                                     ("p2", "c2"): 4},
+                                           ("p1", "c2"): 2,
+                                           ("p2", "c1"): 3,
+                                           ("p2", "c2"): 4},
                   "pressure": 2e5,
                   "temperature": 500}
 
@@ -463,9 +468,9 @@ def test_revert_state_vars_guesses(model):
     # Note that flow_mol_phase_comp is labled as compoennt_flow
     # in define_state_vars
     state_args = {"component_flow_phase": {("p1", "c1"): 1,
-                                     ("p1", "c2"): 2,
-                                     ("p2", "c1"): 3,
-                                     ("p2", "c2"): 4},
+                                           ("p1", "c2"): 2,
+                                           ("p2", "c1"): 3,
+                                           ("p2", "c2"): 4},
                   "pressure": 2e5,
                   "temperature": 500}
 
@@ -522,9 +527,9 @@ def test_revert_state_vars_fixed_guesses(model):
     model.fs.sb.pressure.fix(1.5e5)
 
     state_args = {"component_flow_phase": {("p1", "c1"): 1,
-                                     ("p1", "c2"): 2,
-                                     ("p2", "c1"): 3,
-                                     ("p2", "c2"): 4},
+                                           ("p1", "c2"): 2,
+                                           ("p2", "c1"): 3,
+                                           ("p2", "c2"): 4},
                   "pressure": 2e5,
                   "temperature": 500}
 
@@ -647,7 +652,7 @@ def test_propagate_state_reverse():
 
 
 @pytest.mark.unit
-def test_propagate_state_fixed():
+def test_propagate_state_indexed_fixed():
     m = ConcreteModel()
 
     def block_rule(b):
@@ -655,15 +660,17 @@ def test_propagate_state_fixed():
         b.v1 = Var()
         b.v2 = Var(b.s)
 
-        b.p = Port()
-        b.p.add(b.v1, "V1")
-        b.p.add(b.v2, "V2")
+        b.p = Port(b.s)
+        b.p[1].add(b.v1, "V1")
+        b.p[2].add(b.v2, "V2")
         return
 
     m.b1 = Block(rule=block_rule)
     m.b2 = Block(rule=block_rule)
 
-    m.s1 = Arc(source=m.b1.p, destination=m.b2.p)
+    def arc_rule(m,i):
+        return {'source': m.b1.p[i], 'destination':m.b2.p[i]}
+    m.s1 = Arc([1,2], rule=arc_rule)
 
     # Set values on first block
     m.b1.v1.value = 10
@@ -678,21 +685,49 @@ def test_propagate_state_fixed():
     # Fix v1 in block 2
     m.b2.v1.fix(500)
 
-    propagate_state(m.s1)
+    propagate_state(m.s1[1])
 
     # Check that values were propagated correctly
     assert m.b2.v1.value == 500
+    assert m.b1.v1.fixed is False
+    assert m.b2.v1.fixed is True
+
+    propagate_state(m.s1[2])
+
+    # Check that values were propagated correctly
     assert m.b2.v2[1].value == m.b1.v2[1].value
     assert m.b2.v2[2].value == m.b1.v2[2].value
-
-    assert m.b1.v1.fixed is False
     assert m.b1.v2[1].fixed is False
     assert m.b1.v2[2].fixed is False
-    assert m.b2.v1.fixed is True
     assert m.b2.v2[1].fixed is False
     assert m.b2.v2[2].fixed is False
 
 
+@pytest.mark.unit
+def test_propagate_state_indexed():
+    m = ConcreteModel()
+
+    def block_rule(b):
+        b.s = Set(initialize=[1, 2])
+        b.v1 = Var()
+        b.v2 = Var(b.s)
+
+        b.p = Port(b.s)
+        b.p[1].add(b.v1, "V1")
+        b.p[2].add(b.v2, "V2")
+        return
+
+    m.b1 = Block(rule=block_rule)
+    m.b2 = Block(rule=block_rule)
+
+    def arc_rule(m,i):
+        return {'source': m.b1.p[i], 'destination':m.b2.p[i]}
+    m.s1 = Arc([1,2], rule=arc_rule)
+
+    with pytest.raises(AttributeError):
+        propagate_state(m.s1)
+
+
 @pytest.mark.unit
 def test_propagate_state_Expression():
     m = ConcreteModel()
@@ -714,7 +749,7 @@ def test_propagate_state_Expression():
 
     m.s1 = Arc(source=m.b1.p, destination=m.b2.p)
 
-    with pytest.raises(TypeError):
+    with pytest.raises(ValueError):
         propagate_state(m.s1)
 
 
@@ -722,7 +757,7 @@ def test_propagate_state_Expression():
 def test_propagate_state_invalid_stream():
     m = ConcreteModel()
 
-    with pytest.raises(TypeError):
+    with pytest.raises(RuntimeError):
         propagate_state(m)
 
 
@@ -797,7 +832,7 @@ def test_solve_indexed_block_error():
 def test_initialize_by_time_element():
     horizon = 6
     time_set = [0, horizon]
-    ntfe = 60 # For a finite element every six seconds
+    ntfe = 60  # For a finite element every six seconds
     ntcp = 2
     m = ConcreteModel(name='CSTR model for testing')
     m.fs = FlowsheetBlock(default={'dynamic': True,
@@ -806,16 +841,18 @@ def test_initialize_by_time_element():
     m.fs.properties = AqueousEnzymeParameterBlock()
     m.fs.reactions = EnzymeReactionParameterBlock(
             default={'property_package': m.fs.properties})
-    m.fs.cstr = CSTR(default={"property_package": m.fs.properties,
-                              "reaction_package": m.fs.reactions,
-                              "material_balance_type": MaterialBalanceType.componentTotal,
-                              "energy_balance_type": EnergyBalanceType.enthalpyTotal,
-                              "momentum_balance_type": MomentumBalanceType.none,
-                              "has_heat_of_reaction": True})
+    m.fs.cstr = CSTR(default={
+        "property_package": m.fs.properties,
+        "reaction_package": m.fs.reactions,
+        "material_balance_type": MaterialBalanceType.componentTotal,
+        "energy_balance_type": EnergyBalanceType.enthalpyTotal,
+        "momentum_balance_type": MomentumBalanceType.none,
+        "has_heat_of_reaction": True})
 
     # Time discretization
     disc = TransformationFactory('dae.collocation')
-    disc.apply_to(m, wrt=m.fs.time, nfe=ntfe, ncp=ntcp, scheme='LAGRANGE-RADAU')
+    disc.apply_to(
+        m, wrt=m.fs.time, nfe=ntfe, ncp=ntcp, scheme='LAGRANGE-RADAU')
 
     # Fix geometry variables
     m.fs.cstr.volume[0].fix(1.0)
@@ -877,16 +914,17 @@ def test_initialize_by_time_element():
 
     # Assert that model is still fixed and deactivated as expected
     assert (
-    m.fs.cstr.control_volume.material_holdup[m.fs.time.first(), 'aq', 'S'].fixed)
+        m.fs.cstr.control_volume.material_holdup[
+            m.fs.time.first(), 'aq', 'S'].fixed)
 
     for t in m.fs.time:
         if t != m.fs.time.first():
-            assert (not 
-    m.fs.cstr.control_volume.material_holdup[t, 'aq', 'S'].fixed)
+            assert (not m.fs.cstr.control_volume.material_holdup[
+                t, 'aq', 'S'].fixed)
 
             assert not m.fs.cstr.outlet.temperature[t].fixed
-        assert (
-    m.fs.cstr.control_volume.material_holdup_calculation[t, 'aq', 'C'].active)
+        assert (m.fs.cstr.control_volume.material_holdup_calculation[
+            t, 'aq', 'C'].active)
 
         assert m.fs.cstr.control_volume.properties_out[t].active
         assert not m.fs.cstr.outlet.conc_mol[t, 'S'].fixed
@@ -899,6 +937,3 @@ def test_initialize_by_time_element():
 
     results = solver.solve(m.fs)
     assert results.solver.termination_condition == TerminationCondition.optimal
-
-if __name__ == '__main__':
-    test_initialize_by_time_element()
