@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 Test surrogate modeling.
 """
@@ -24,7 +24,7 @@ from typing import Union
 import pytest
 
 # local
-from idaes.dmf import surrmod, errors, DMF
+from idaes.dmf import surrmod, DMF
 from idaes.dmf.experiment import Experiment
 
 # for testing
@@ -82,7 +82,7 @@ class AlamoMock(object):
 def test_init():
     tmp_dir = scratch_path / "init"
     dmf = DMF(path=tmp_dir, create=True)
-    surrmod.SurrogateModel(Experiment(DMF))
+    surrmod.SurrogateModel(Experiment(dmf))
 
 
 @pytest.mark.skipif(not surrmod.alamo_enabled, reason="ALAMO is disabled")
