@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 Ideal gas + Ideal/Non-ideal liquid property package.
 
@@ -43,7 +43,7 @@ References:
 
 # Import Pyomo libraries
 from pyomo.environ import Constraint, log, NonNegativeReals, value, Var, exp,\
-    Expression, Param, sqrt, SolverFactory
+    Expression, Param, sqrt, SolverFactory, units as pyunits
 from pyomo.common.config import ConfigValue, In
 
 # Import IDAES cores
@@ -62,6 +62,7 @@ from idaes.core.util.initialization import (fix_state_vars,
 from idaes.core.util.exceptions import ConfigurationError
 from idaes.core.util.model_statistics import degrees_of_freedom
 from idaes.core.util.constants import Constants as const
+from idaes.core.util import get_solver
 import idaes.logger as idaeslog
 
 
@@ -160,7 +161,8 @@ conditions, and thus corresponding constraints  should be included,
                 # user or need to be estimated based on VLE data)
                 self.vol_mol_comp = Var(self.component_list,
                                         initialize=1.0,
-                                        doc="Molar volume of component")
+                                        doc="Molar volume of component",
+                                        units=pyunits.m**3/pyunits.mol)
 
                 self.tau = Var(self.component_list,
                                self.component_list,
@@ -210,13 +212,11 @@ conditions, and thus corresponding constraints  should be included,
              "fug_liq": {"method": "_fug_liq", "units": "Pa"},
              'ds_form': {'method': '_ds_form', 'units': 'J/mol.K'}})
 
-        obj.add_default_units({"time": "s",
-                               "length": "m",
-                               "mass": "g",
-                               "amount": "mol",
-                               "temperature": "K",
-                               "energy": "J",
-                               "holdup": "mol"})
+        obj.add_default_units({"time": pyunits.s,
+                               "length": pyunits.m,
+                               "mass": pyunits.kg,
+                               "amount": pyunits.mol,
+                               "temperature": pyunits.K})
 
 
 class _ActivityCoeffStateBlock(StateBlock):
@@ -225,9 +225,9 @@ class _ActivityCoeffStateBlock(StateBlock):
     whole, rather than individual elements of indexed Property Blocks.
     """
 
-    def initialize(blk, state_args={}, hold_state=False,
+    def initialize(blk, state_args=None, hold_state=False,
                    state_vars_fixed=False, outlvl=idaeslog.NOTSET,
-                   solver="ipopt", optarg={"tol": 1e-8}):
+                   solver=None, optarg=None):
         """
         Initialization routine for property package.
         Keyword Arguments:
@@ -247,9 +247,10 @@ class _ActivityCoeffStateBlock(StateBlock):
                          flow_mol_comp, temperature, pressure.
 
             outlvl : sets output level of initialization routine
-            optarg : solver options dictionary object (default=None)
-            solver : str indicating whcih solver to use during
-                     initialization (default = "ipopt")
+            optarg : solver options dictionary object (default=None, use
+                     default solver options)
+            solver : str indicating which solver to use during
+                     initialization (default = None, use default solve)
             hold_state : flag indicating whether the initialization routine
                          should unfix any state variables fixed during
                          initialization (default=False).
@@ -293,13 +294,8 @@ class _ActivityCoeffStateBlock(StateBlock):
                                     "for state block is not zero during "
                                     "initialization.")
 
-        if optarg is None:
-            sopt = {"tol": 1e-8}
-        else:
-            sopt = optarg
-
-        opt = SolverFactory("ipopt")
-        opt.options = sopt
+        # Create solver
+        opt = get_solver(solver, optarg)
 
         # ---------------------------------------------------------------------
         # Initialization sequence: Deactivating certain constraints
@@ -469,7 +465,8 @@ class ActivityCoeffStateBlockData(StateBlockData):
         if self.params.config.state_vars == "FTPz":
             self.flow_mol = Var(initialize=1.0,
                                 domain=NonNegativeReals,
-                                doc="Total molar flowrate [mol/s]")
+                                doc="Total molar flowrate [mol/s]",
+                                units=pyunits.mol/pyunits.s)
             self.mole_frac_comp = Var(
                     self.params.component_list,
                     bounds=(0, 1),
@@ -477,32 +474,38 @@ class ActivityCoeffStateBlockData(StateBlockData):
                     doc="Mixture mole fraction")
             self.pressure = Var(initialize=101325,
                                 domain=NonNegativeReals,
-                                doc="State pressure [Pa]")
+                                doc="State pressure [Pa]",
+                                units=pyunits.Pa)
             self.temperature = Var(initialize=298.15,
                                    domain=NonNegativeReals,
-                                   doc="State temperature [K]")
+                                   doc="State temperature [K]",
+                                   units=pyunits.K)
         else:
             self.flow_mol_comp = Var(
                     self.params.component_list,
                     initialize=1 / len(self.params.component_list),
                     domain=NonNegativeReals,
-                    doc="Component molar flowrate [mol/s]")
+                    doc="Component molar flowrate [mol/s]",
+                    units=pyunits.mol/pyunits.s)
             self.pressure = Var(initialize=101325,
                                 domain=NonNegativeReals,
-                                doc="State pressure [Pa]")
+                                doc="State pressure [Pa]",
+                                units=pyunits.Pa)
             self.temperature = Var(initialize=298.15,
                                    domain=NonNegativeReals,
-                                   doc="State temperature [K]")
+                                   doc="State temperature [K]",
+                                   units=pyunits.K)
 
     def _make_vars(self):
 
         if self.params.config.state_vars == "FTPz":
             self.flow_mol_phase = Var(self.params.phase_list,
-                                      initialize=0.5)
+                                      initialize=0.5,
+                                      units=pyunits.mol/pyunits.s)
         else:
-            self.flow_mol_phase_comp = Var(self.params.phase_list,
-                                           self.params.component_list,
-                                           initialize=0.5)
+            self.flow_mol_phase_comp = Var(self.params._phase_component_set,
+                                           initialize=0.5,
+                                           units=pyunits.mol/pyunits.s)
 
             def rule_mix_mole_frac(self, i):
                 return self.flow_mol_comp[i] / \
@@ -512,8 +515,7 @@ class ActivityCoeffStateBlockData(StateBlockData):
                                              rule=rule_mix_mole_frac)
 
         self.mole_frac_phase_comp = \
-            Var(self.params.phase_list,
-                self.params.component_list,
+            Var(self.params._phase_component_set,
                 initialize=1 / len(self.params.component_list),
                 bounds=(0, 1))
 
@@ -631,8 +633,7 @@ class ActivityCoeffStateBlockData(StateBlockData):
                     sum(self.flow_mol_phase_comp[p, i]
                         for i in self.params.component_list) == \
                     self.flow_mol_phase_comp[p, i]
-            self.eq_mole_frac = Constraint(self.params.phase_list,
-                                           self.params.component_list,
+            self.eq_mole_frac = Constraint(self.params._phase_component_set,
                                            rule=rule_mole_frac)
 
         # Smooth Flash Formulation
@@ -644,20 +645,24 @@ class ActivityCoeffStateBlockData(StateBlockData):
         self._temperature_equilibrium = \
             Var(initialize=self.temperature.value,
                 doc="Temperature for calculating "
-                    "phase equilibrium")
+                    "phase equilibrium",
+                units=pyunits.K)
 
         self._t1 = Var(initialize=self.temperature.value,
                        doc="Intermediate temperature for calculating "
-                           "the equilibrium temperature")
+                           "the equilibrium temperature",
+                       units=pyunits.K)
 
         self.eps_1 = Param(default=0.01,
                            mutable=True,
                            doc="Smoothing parameter for equilibrium "
-                               "temperature")
+                               "temperature",
+                           units=pyunits.K)
         self.eps_2 = Param(default=0.0005,
                            mutable=True,
                            doc="Smoothing parameter for equilibrium "
-                               "temperature")
+                               "temperature",
+                           units=pyunits.K)
 
         # Equation #13 in reference cited above
         # Approximation for max(temperature, temperature_bubble)
@@ -819,7 +824,8 @@ class ActivityCoeffStateBlockData(StateBlockData):
     def _pressure_sat(self):
         self.pressure_sat = Var(self.params.component_list,
                                 initialize=101325,
-                                doc="vapor pressure ")
+                                doc="vapor pressure",
+                                units=pyunits.Pa)
 
         def rule_reduced_temp(self, i):
             # reduced temperature is variable "x" in the documentation
@@ -863,7 +869,9 @@ class ActivityCoeffStateBlockData(StateBlockData):
                                   rule=rule_fug_liq)
 
     def _density_mol(self):
-        self.density_mol = Var(self.params.phase_list, doc="Molar density")
+        self.density_mol = Var(self.params.phase_list,
+                               doc="Molar density",
+                               units=pyunits.mol/pyunits.m**3)
 
         def density_mol_calculation(self, p):
             if p == "Vap":
@@ -889,7 +897,8 @@ class ActivityCoeffStateBlockData(StateBlockData):
     def _energy_internal_mol_phase(self):
         self.energy_internal_mol_phase = Var(
             self.params.phase_list,
-            doc="Phase molar specific internal energy [J/mol]")
+            doc="Phase molar specific internal energy [J/mol]",
+            units=pyunits.J/pyunits.mol)
 
         def rule_energy_internal_mol_phase(b, p):
             return b.energy_internal_mol_phase[p] == sum(
@@ -902,9 +911,9 @@ class ActivityCoeffStateBlockData(StateBlockData):
 
     def _energy_internal_mol_phase_comp(self):
         self.energy_internal_mol_phase_comp = Var(
-            self.params.phase_list,
-            self.params.component_list,
-            doc="Phase-component molar specific internal energy [J/mol]")
+            self.params._phase_component_set,
+            doc="Phase-component molar specific internal energy [J/mol]",
+            units=pyunits.J/pyunits.mol)
 
         def rule_energy_internal_mol_phase_comp(b, p, j):
             if p == "Vap":
@@ -916,14 +925,14 @@ class ActivityCoeffStateBlockData(StateBlockData):
                 return b.energy_internal_mol_phase_comp[p, j] == \
                     b.enth_mol_phase_comp[p, j]
         self.eq_energy_internal_mol_phase_comp = Constraint(
-            self.params.phase_list,
-            self.params.component_list,
+            self.params._phase_component_set,
             rule=rule_energy_internal_mol_phase_comp)
 
     def _enth_mol_phase(self):
         self.enth_mol_phase = Var(
             self.params.phase_list,
-            doc="Phase molar specific enthalpies [J/mol]")
+            doc="Phase molar specific enthalpies [J/mol]",
+            units=pyunits.J/pyunits.mol)
 
         def rule_enth_mol_phase(b, p):
             return b.enth_mol_phase[p] == sum(
@@ -934,10 +943,10 @@ class ActivityCoeffStateBlockData(StateBlockData):
                                             rule=rule_enth_mol_phase)
 
     def _enth_mol_phase_comp(self):
-        self.enth_mol_phase_comp = Var(self.params.phase_list,
-                                       self.params.component_list,
+        self.enth_mol_phase_comp = Var(self.params._phase_component_set,
                                        doc="Phase-component molar specific "
-                                           "enthalpies [J/mol]")
+                                           "enthalpies [J/mol]",
+                                       units=pyunits.J/pyunits.mol)
 
         def rule_enth_mol_phase_comp(b, p, j):
             if p == "Vap":
@@ -945,55 +954,57 @@ class ActivityCoeffStateBlockData(StateBlockData):
             else:
                 return b._enth_mol_comp_liq(j)
         self.eq_enth_mol_phase_comp = Constraint(
-            self.params.phase_list,
-            self.params.component_list,
+            self.params._phase_component_set,
             rule=rule_enth_mol_phase_comp)
 
     def _enth_mol_comp_liq(self, j):
         # Liquid phase comp enthalpy (J/mol)
         # 1E3 conversion factor to convert from J/kmol to J/mol
-        return self.enth_mol_phase_comp["Liq", j] * 1E3 == \
-            1e3*self.params.dh_form["Liq", j] + \
-            ((self.params.CpIG["Liq", j, "E"] / 5) *
+        return self.enth_mol_phase_comp["Liq", j] == \
+            self.params.dh_form["Liq", j] + \
+            pyunits.convert((
+                (self.params.cp_mol_liq_comp_coeff_E[j] / 5) *
                 (self.temperature**5 -
                  self.params.temperature_reference**5)
-                + (self.params.CpIG["Liq", j, "D"] / 4) *
+                + (self.params.cp_mol_liq_comp_coeff_D[j] / 4) *
                   (self.temperature**4 -
                    self.params.temperature_reference**4)
-                + (self.params.CpIG["Liq", j, "C"] / 3) *
+                + (self.params.cp_mol_liq_comp_coeff_C[j] / 3) *
                   (self.temperature**3 -
                    self.params.temperature_reference**3)
-                + (self.params.CpIG["Liq", j, "B"] / 2) *
+                + (self.params.cp_mol_liq_comp_coeff_B[j] / 2) *
                   (self.temperature**2 -
                    self.params.temperature_reference**2)
-                + self.params.CpIG["Liq", j, "A"] *
-                  (self.temperature - self.params.temperature_reference))
+                + self.params.cp_mol_liq_comp_coeff_A[j] *
+                  (self.temperature - self.params.temperature_reference)),
+                to_units=pyunits.J/pyunits.mol)
 
     def _enth_mol_comp_vap(self, j):
 
         # Vapor phase component enthalpy (J/mol)
         return self.enth_mol_phase_comp["Vap", j] == \
             self.params.dh_form["Vap", j] + \
-            ((self.params.CpIG["Vap", j, "E"] / 5) *
+            ((self.params.cp_mol_vap_comp_coeff_E[j] / 5) *
                 (self.temperature**5 -
                  self.params.temperature_reference**5)
-                + (self.params.CpIG["Vap", j, "D"] / 4) *
+                + (self.params.cp_mol_vap_comp_coeff_D[j] / 4) *
                   (self.temperature**4 -
                    self.params.temperature_reference**4)
-                + (self.params.CpIG["Vap", j, "C"] / 3) *
+                + (self.params.cp_mol_vap_comp_coeff_C[j] / 3) *
                   (self.temperature**3 -
                    self.params.temperature_reference**3)
-                + (self.params.CpIG["Vap", j, "B"] / 2) *
+                + (self.params.cp_mol_vap_comp_coeff_B[j] / 2) *
                   (self.temperature**2 -
                    self.params.temperature_reference**2)
-                + self.params.CpIG["Vap", j, "A"] *
+                + self.params.cp_mol_vap_comp_coeff_A[j] *
                   (self.temperature -
                    self.params.temperature_reference))
 
     def _entr_mol_phase(self):
         self.entr_mol_phase = Var(
             self.params.phase_list,
-            doc="Phase molar specific enthropies [J/mol.K]")
+            doc="Phase molar specific enthropies [J/mol.K]",
+            units=pyunits.J/pyunits.mol/pyunits.K)
 
         def rule_entr_mol_phase(self, p):
             return self.entr_mol_phase[p] == sum(
@@ -1005,9 +1016,9 @@ class ActivityCoeffStateBlockData(StateBlockData):
 
     def _entr_mol_phase_comp(self):
         self.entr_mol_phase_comp = Var(
-            self.params.phase_list,
-            self.params.component_list,
-            doc="Phase-component molar specific entropies [J/mol.K]")
+            self.params._phase_component_set,
+            doc="Phase-component molar specific entropies [J/mol.K]",
+            units=pyunits.J/pyunits.mol/pyunits.K)
 
         def rule_entr_mol_phase_comp(self, p, j):
             if p == "Vap":
@@ -1015,39 +1026,40 @@ class ActivityCoeffStateBlockData(StateBlockData):
             else:
                 return self._entr_mol_comp_liq(j)
         self.eq_entr_mol_phase_comp = Constraint(
-            self.params.phase_list,
-            self.params.component_list,
+            self.params._phase_component_set,
             rule=rule_entr_mol_phase_comp)
 
     def _entr_mol_comp_liq(self, j):
         # Liquid phase comp entropy (J/mol.K)
         # 1E3 conversion factor to convert from J/kmol.K to J/mol.K
-        return self.entr_mol_phase_comp['Liq', j] * 1E3 == \
-            1E3*self.params.ds_form["Liq", j] + (
-            ((self.params.CpIG['Liq', j, 'E'] / 4) *
+        return self.entr_mol_phase_comp['Liq', j] == (
+            self.params.ds_form["Liq", j] +
+            pyunits.convert((
+                (self.params.cp_mol_liq_comp_coeff_E[j] / 4) *
                 (self.temperature**4 - self.params.temperature_reference**4)
-                + (self.params.CpIG['Liq', j, 'D'] / 3) *
+                + (self.params.cp_mol_liq_comp_coeff_D[j] / 3) *
                   (self.temperature**3 - self.params.temperature_reference**3)
-                + (self.params.CpIG['Liq', j, 'C'] / 2) *
+                + (self.params.cp_mol_liq_comp_coeff_C[j] / 2) *
                   (self.temperature**2 - self.params.temperature_reference**2)
-                + self.params.CpIG['Liq', j, 'B'] *
+                + self.params.cp_mol_liq_comp_coeff_B[j] *
                   (self.temperature - self.params.temperature_reference)
-                + self.params.CpIG['Liq', j, 'A'] *
-             log(self.temperature / self.params.temperature_reference)))
+                + self.params.cp_mol_liq_comp_coeff_A[j] *
+                log(self.temperature / self.params.temperature_reference)),
+                to_units=pyunits.J/pyunits.mol/pyunits.K))
 
     def _entr_mol_comp_vap(self, j):
         # component molar entropy of vapor phase
         return self.entr_mol_phase_comp["Vap", j] == (
             self.params.ds_form["Vap", j] +
-            ((self.params.CpIG['Vap', j, 'E'] / 4) *
+            ((self.params.cp_mol_vap_comp_coeff_E[j] / 4) *
              (self.temperature**4 - self.params.temperature_reference**4)
-             + (self.params.CpIG['Vap', j, 'D'] / 3) *
+             + (self.params.cp_mol_vap_comp_coeff_D[j] / 3) *
              (self.temperature**3 - self.params.temperature_reference**3)
-             + (self.params.CpIG['Vap', j, 'C'] / 2) *
+             + (self.params.cp_mol_vap_comp_coeff_C[j] / 2) *
                (self.temperature**2 - self.params.temperature_reference**2)
-                + self.params.CpIG['Vap', j, 'B'] *
+                + self.params.cp_mol_vap_comp_coeff_B[j] *
                (self.temperature - self.params.temperature_reference)
-                + self.params.CpIG['Vap', j, 'A'] *
+                + self.params.cp_mol_vap_comp_coeff_A[j] *
                 log(self.temperature / self.params.temperature_reference)) -
             const.gas_constant * log(self.mole_frac_phase_comp['Vap', j] *
                                      self.pressure /
@@ -1055,17 +1067,16 @@ class ActivityCoeffStateBlockData(StateBlockData):
 
     def _gibbs_mol_phase_comp(self):
         self.gibbs_mol_phase_comp = Var(
-            self.params.phase_list,
-            self.params.component_list,
-            doc="Phase-component molar specific Gibbs energies [J/mol]")
+            self.params._phase_component_set,
+            doc="Phase-component molar specific Gibbs energies [J/mol]",
+            units=pyunits.J/pyunits.mol)
 
         def rule_gibbs_mol_phase_comp(self, p, j):
             return self.gibbs_mol_phase_comp[p, j] == \
                     self.enth_mol_phase_comp[p, j] - \
                     self.temperature*self.entr_mol_phase_comp[p, j]
         self.eq_gibbs_mol_phase_comp = Constraint(
-            self.params.phase_list,
-            self.params.component_list,
+            self.params._phase_component_set,
             rule=rule_gibbs_mol_phase_comp)
 
     def get_material_flow_terms(self, p, j):
@@ -1159,7 +1170,8 @@ class ActivityCoeffStateBlockData(StateBlockData):
 # Bubble and Dew Points
     def _temperature_bubble(self):
         self.temperature_bubble = Var(initialize=298.15,
-                                      doc="Bubble point temperature (K)")
+                                      doc="Bubble point temperature (K)",
+                                      units=pyunits.K)
 
         def rule_psat_bubble(m, j):
             return self.params.pressure_critical[j] * \
@@ -1305,7 +1317,8 @@ class ActivityCoeffStateBlockData(StateBlockData):
     def _temperature_dew(self):
 
         self.temperature_dew = Var(initialize=298.15,
-                                   doc="Dew point temperature (K)")
+                                   doc="Dew point temperature (K)",
+                                   units=pyunits.K)
 
         def rule_psat_dew(m, j):
             return self.params.pressure_critical[j] * \
