@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 Tests for Cubic equation of state methods
 
@@ -25,7 +25,8 @@ from pyomo.environ import (ConcreteModel,
                            Param,
                            sqrt,
                            value,
-                           Var)
+                           Var,
+                           units as pyunits)
 from pyomo.core.expr.numeric_expr import ExternalFunctionExpression
 
 from idaes.core import (declare_process_block_class,
@@ -49,9 +50,10 @@ def set_metadata(b):
 
 
 # Dummy methods for dummied submodules
-def dummy_pe(b, *args):
-    # Return a dummy expression for the constraint
-    return b.temperature == 100
+class dummy_pe():
+    def return_expression(b, *args):
+        # Return a dummy expression for the constraint
+        return b.temperature == 100
 
 
 def phase_equil(b, *args):
@@ -116,6 +118,11 @@ def m():
                             "equation_of_state": Cubic,
                             "equation_of_state_options": {
                                 "type": CubicType.PR}}},
+                "base_units": {"time": pyunits.s,
+                               "length": pyunits.m,
+                               "mass": pyunits.kg,
+                               "amount": pyunits.mol,
+                               "temperature": pyunits.K},
                 "state_definition": modules[__name__],
                 "pressure_ref": 1e5,
                 "temperature_ref": 300,
@@ -173,6 +180,11 @@ def m_sol():
                             "equation_of_state": Cubic,
                             "equation_of_state_options": {
                                 "type": CubicType.PR}}},
+                "base_units": {"time": pyunits.s,
+                               "length": pyunits.m,
+                               "mass": pyunits.kg,
+                               "amount": pyunits.mol,
+                               "temperature": pyunits.K},
                 "state_definition": modules[__name__],
                 "pressure_ref": 1e5,
                 "temperature_ref": 300,
@@ -209,6 +221,10 @@ Bl_eq = 0.0788309
 Av_eq = 1.9805792
 Bv_eq = 0.0788309
 
+Hl = -212.211
+Hv = -779.305
+Ul = Hl - 8.314*300*(Zl-1)
+Uv = Hv - 8.314*300*(Zv-1)
 
 # Set path to root finder .so file
 _so = os.path.join(bin_directory, "cubic_roots.so")
@@ -429,11 +445,10 @@ def test_dens_mass_phase(m):
 
 @pytest.mark.unit
 def test_dens_mol_phase(m):
-    for p in m.params.phase_list:
-        assert str(Cubic.dens_mol_phase(m.props[1], p)) == (
-                str(m.props[1].pressure)+"/(8.314462618*J/mol/K*" +
-                str(m.props[1].temperature *
-                    m.props[1].compress_fact_phase[p])+")")
+    assert value(Cubic.dens_mol_phase(m.props[1], "Vap")) == pytest.approx(
+            44.800, rel=1e-3)
+    assert value(Cubic.dens_mol_phase(m.props[1], "Liq")) == pytest.approx(
+            41.157, rel=1e-3)
 
 
 @pytest.mark.unit
@@ -442,6 +457,41 @@ def test_dens_mol_phase_invalid_phase(m_sol):
         Cubic.dens_mol_phase(m_sol.props[1], "Sol")
 
 
+@pytest.mark.unit
+def test_energy_internal_mol_phase(m):
+    for j in m.params.component_list:
+        m.params.config.include_enthalpy_of_formation = False
+        m.params.get_component(j).config.enth_mol_liq_comp = dummy_call
+        m.params.get_component(j).config.enth_mol_ig_comp = dummy_call
+
+    m.props[1].energy_internal_mol_phase_comp = Var(
+        m.params.phase_list, m.params.component_list, initialize=1)
+
+    assert pytest.approx(Uv, rel=1e-4) == value(
+        Cubic.energy_internal_mol_phase(m.props[1], "Vap"))
+    assert pytest.approx(Ul, rel=1e-4) == value(
+        Cubic.energy_internal_mol_phase(m.props[1], "Liq"))
+
+
+@pytest.mark.unit
+def test_energy_internal_mol_phase_comp(m):
+    for j in m.params.component_list:
+        m.params.config.include_enthalpy_of_formation = False
+        m.params.get_component(j).config.enth_mol_liq_comp = dummy_call
+        m.params.get_component(j).config.enth_mol_ig_comp = dummy_call
+
+        assert pytest.approx(Ul, rel=1e-4) == value(
+            Cubic.energy_internal_mol_phase_comp(m.props[1], "Liq", j))
+        assert pytest.approx(Uv, rel=1e-4) == value(
+            Cubic.energy_internal_mol_phase_comp(m.props[1], "Vap", j))
+
+
+@pytest.mark.unit
+def test_energy_internal_mol_phase_invalid_phase(m_sol):
+    with pytest.raises(PropertyNotSupportedError):
+        Cubic.energy_internal_mol_phase_comp(m_sol.props[1], "Sol", "foo")
+
+
 @pytest.mark.unit
 def test_enth_mol_phase(m):
     for j in m.params.component_list:
@@ -453,9 +503,9 @@ def test_enth_mol_phase(m):
                                          initialize=1)
 
     assert pytest.approx(value(
-        Cubic.enth_mol_phase(m.props[1], "Vap")), rel=1e-5) == -779.31
+        Cubic.enth_mol_phase(m.props[1], "Vap")), rel=1e-5) == Hv
     assert pytest.approx(value(
-        Cubic.enth_mol_phase(m.props[1], "Liq")), rel=1e-5) == -212.211
+        Cubic.enth_mol_phase(m.props[1], "Liq")), rel=1e-5) == Hl
 
 
 @pytest.mark.unit
@@ -464,9 +514,9 @@ def test_enth_mol_phase_comp(m):
         m.params.get_component(j).config.enth_mol_liq_comp = dummy_call
         m.params.get_component(j).config.enth_mol_ig_comp = dummy_call
 
-        assert pytest.approx(-212.211, rel=1e-5) == value(
+        assert pytest.approx(Hl, rel=1e-5) == value(
             Cubic.enth_mol_phase_comp(m.props[1], "Liq", j))
-        assert pytest.approx(-779.305, rel=1e-5) == value(
+        assert pytest.approx(Hv, rel=1e-5) == value(
             Cubic.enth_mol_phase_comp(m.props[1], "Vap", j))
 
 
@@ -628,3 +678,17 @@ def test_gibbs_mol_phase_comp(m):
                     m.props[1].enth_mol_phase_comp[p, j] -
                     m.props[1].entr_mol_phase_comp[p, j] *
                     m.props[1].temperature)
+
+
+@pytest.mark.unit
+def test_vol_mol_phase(m):
+    assert value(Cubic.vol_mol_phase(m.props[1], "Vap")) == pytest.approx(
+            1/44.800, rel=1e-3)
+    assert value(Cubic.vol_mol_phase(m.props[1], "Liq")) == pytest.approx(
+            1/41.157, rel=1e-3)
+
+
+@pytest.mark.unit
+def test_vol_mol_phase_invalid_phase(m_sol):
+    with pytest.raises(PropertyNotSupportedError):
+        Cubic.vol_mol_phase(m_sol.props[1], "Sol")
