@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 Tests for generic reaction package core code
 
@@ -19,29 +19,31 @@ import pytest
 from sys import modules
 
 from pyomo.environ import (
-    Block, ConcreteModel, Constraint, Expression, exp, Set, Var, value)
+    Block, ConcreteModel, Constraint, Expression,
+    exp, Set, Var, value, units as pyunits)
 
 from idaes.generic_models.properties.core.generic.generic_property import (
         GenericParameterBlock)
-from idaes.generic_models.properties.core.generic.tests import dummy_eos
+from idaes.generic_models.properties.core.generic.tests.dummy_eos import DummyEoS
 
 from idaes.generic_models.properties.core.generic.generic_reaction import (
-        GenericReactionParameterBlock)
+        GenericReactionParameterBlock, ConcentrationForm)
 from idaes.generic_models.properties.core.reactions.dh_rxn import \
     constant_dh_rxn
 from idaes.generic_models.properties.core.reactions.rate_constant import \
     arrhenius
 from idaes.generic_models.properties.core.reactions.rate_forms import \
-    mole_frac_power_law_rate
+    power_law_rate
 from idaes.generic_models.properties.core.reactions.equilibrium_constant import \
     van_t_hoff
 from idaes.generic_models.properties.core.reactions.equilibrium_forms import \
-    mole_frac_power_law_equil
+    power_law_equil
 
 from idaes.core.util.testing import PhysicalParameterTestBlock
 from idaes.core.util.constants import Constants as constants
 
 from idaes.core.util.exceptions import ConfigurationError
+import idaes.logger as idaeslog
 
 
 # -----------------------------------------------------------------------------
@@ -57,6 +59,14 @@ def define_state(b):
                                  initialize=0.5)
 
 
+# Declare a base units dict to save code later
+base_units = {"time": pyunits.s,
+              "length": pyunits.m,
+              "mass": pyunits.kg,
+              "amount": pyunits.mol,
+              "temperature": pyunits.K}
+
+
 # -----------------------------------------------------------------------------
 class TestGenericReactionParameterBlock(object):
     @pytest.fixture
@@ -67,11 +77,12 @@ class TestGenericReactionParameterBlock(object):
         m.params = GenericParameterBlock(default={
                 "components": {"c1": {}, "c2": {}},
                 "phases": {
-                    "p1": {"equation_of_state": dummy_eos},
-                    "p2": {"equation_of_state": dummy_eos}},
+                    "p1": {"equation_of_state": DummyEoS},
+                    "p2": {"equation_of_state": DummyEoS}},
                 "state_definition": modules[__name__],
                 "pressure_ref": 1e5,
-                "temperature_ref": 300})
+                "temperature_ref": 300,
+                "base_units": base_units})
 
         return m
 
@@ -83,10 +94,20 @@ class TestGenericReactionParameterBlock(object):
                 "r1": {"stoichiometry": {("p1", "c1"): -1,
                                          ("p1", "c2"): 2},
                        "heat_of_reaction": "foo",
-                       "rate_form": "foo"}}})
+                       "rate_form": "foo"}},
+            "base_units": base_units})
 
         rxn_config = m.rxn_params.config.rate_reactions
 
+        assert m.rxn_params.get_metadata().default_units == {
+            "time": pyunits.s,
+            "length": pyunits.m,
+            "mass": pyunits.kg,
+            "amount": pyunits.mol,
+            "temperature": pyunits.K,
+            "current": None,
+            "luminous intensity": None}
+
         assert isinstance(m.rxn_params.rate_reaction_idx, Set)
         assert len(m.rxn_params.rate_reaction_idx) == 1
         assert "r1" in m.rxn_params.rate_reaction_idx
@@ -107,7 +128,45 @@ class TestGenericReactionParameterBlock(object):
         assert m.rxn_params.reaction_idx == m.rxn_params.rate_reaction_idx
 
         assert isinstance(m.rxn_params.reaction_r1, Block)
-        
+
+    @pytest.mark.unit
+    def test_invalid_unit(self, m):
+        with pytest.raises(
+                ConfigurationError,
+                match="rxn_params recieved unexpected units for quantity time:"
+                " foo. Units must be instances of a Pyomo unit object."):
+            m.rxn_params = GenericReactionParameterBlock(default={
+                "property_package": m.params,
+                "rate_reactions": {
+                    "r1": {"stoichiometry": {("p1", "c1"): -1,
+                                             ("p1", "c2"): 2},
+                           "heat_of_reaction": "foo",
+                           "rate_form": "foo"}},
+                "base_units": {"time": "foo",
+                               "length": pyunits.m,
+                               "mass": pyunits.kg,
+                               "amount": pyunits.mol,
+                               "temperature": pyunits.K}})
+
+    @pytest.mark.unit
+    def test_missing_required_quantity(self, m):
+        with pytest.raises(
+                ConfigurationError,
+                match="rxn_params units for quantity time were not assigned. "
+                "Please make sure to provide units for all base units "
+                "when configuring the reaction package."):
+            m.rxn_params = GenericReactionParameterBlock(default={
+                "property_package": m.params,
+                "rate_reactions": {
+                    "r1": {"stoichiometry": {("p1", "c1"): -1,
+                                             ("p1", "c2"): 2},
+                           "heat_of_reaction": "foo",
+                           "rate_form": "foo"}},
+                "base_units": {"length": pyunits.m,
+                               "mass": pyunits.kg,
+                               "amount": pyunits.mol,
+                               "temperature": pyunits.K}})
+
     @pytest.mark.unit
     def test_rate_build_no_stoichiometry(self, m):
         with pytest.raises(ConfigurationError,
@@ -116,6 +175,7 @@ class TestGenericReactionParameterBlock(object):
                            "argument."):
             m.rxn_params = GenericReactionParameterBlock(default={
                 "property_package": m.params,
+                "base_units": base_units,
                 "rate_reactions": {
                     "r1": {"heat_of_reaction": "foo",
                            "rate_form": "foo"}}})
@@ -127,6 +187,7 @@ class TestGenericReactionParameterBlock(object):
                            "r1 included unrecognised phase p7."):
             m.rxn_params = GenericReactionParameterBlock(default={
                 "property_package": m.params,
+                "base_units": base_units,
                 "rate_reactions": {
                     "r1": {"stoichiometry": {("p7", "c1"): -1,
                                              ("p1", "c2"): 2},
@@ -140,6 +201,7 @@ class TestGenericReactionParameterBlock(object):
                            "r1 included unrecognised component c7."):
             m.rxn_params = GenericReactionParameterBlock(default={
                 "property_package": m.params,
+                "base_units": base_units,
                 "rate_reactions": {
                     "r1": {"stoichiometry": {("p1", "c7"): -1,
                                              ("p1", "c2"): 2},
@@ -147,22 +209,31 @@ class TestGenericReactionParameterBlock(object):
                            "rate_form": "foo"}}})
 
     @pytest.mark.unit
-    def test_rate_build_no_form(self, m):
-        with pytest.raises(ConfigurationError,
-                           match="rxn_params rate reaction r1 was not "
-                           "provided with a rate_form configuration "
-                           "argument."):
-            m.rxn_params = GenericReactionParameterBlock(default={
-                "property_package": m.params,
-                "rate_reactions": {
-                    "r1": {"stoichiometry": {("p1", "c1"): -1,
-                                             ("p1", "c2"): 2},
-                           "heat_of_reaction": "foo"}}})
-            
+    def test_rate_build_no_form(self, m, caplog):
+        caplog.set_level(
+            idaeslog.DEBUG,
+            logger=("idaes.generic_models.properties.core."
+                    "generic.generic_reaction"))
+
+        m.rxn_params = GenericReactionParameterBlock(default={
+            "property_package": m.params,
+            "base_units": base_units,
+            "rate_reactions": {
+                "r1": {"stoichiometry": {("p1", "c1"): -1,
+                                         ("p1", "c2"): 2},
+                       "heat_of_reaction": "foo"}}})
+
+        assert ("rxn_params rate reaction r1 was not provided with a "
+                "rate_form configuration argument. This is suitable for "
+                "processes using stoichiometric reactors, but not for those "
+                "using unit operations which rely on reaction rate."
+                in caplog.text)
+
     @pytest.mark.unit
     def test_equil_build(self, m):
         m.rxn_params = GenericReactionParameterBlock(default={
             "property_package": m.params,
+            "base_units": base_units,
             "equilibrium_reactions": {
                 "e1": {"stoichiometry": {("p2", "c1"): -3,
                                          ("p2", "c2"): 4},
@@ -202,6 +273,7 @@ class TestGenericReactionParameterBlock(object):
                            "argument."):
             m.rxn_params = GenericReactionParameterBlock(default={
                 "property_package": m.params,
+                "base_units": base_units,
                 "equilibrium_reactions": {
                     "e1": {"heat_of_reaction": "foo",
                            "equilibrium_form": "foo"}}})
@@ -213,6 +285,7 @@ class TestGenericReactionParameterBlock(object):
                            "reaction e1 included unrecognised phase p7."):
             m.rxn_params = GenericReactionParameterBlock(default={
                 "property_package": m.params,
+                "base_units": base_units,
                 "equilibrium_reactions": {
                     "e1": {"stoichiometry": {("p7", "c1"): -3,
                                              ("p2", "c2"): 4},
@@ -226,6 +299,7 @@ class TestGenericReactionParameterBlock(object):
                            "reaction e1 included unrecognised component c7."):
             m.rxn_params = GenericReactionParameterBlock(default={
                 "property_package": m.params,
+                "base_units": base_units,
                 "equilibrium_reactions": {
                     "e1": {"stoichiometry": {("p2", "c7"): -3,
                                              ("p2", "c2"): 4},
@@ -240,6 +314,7 @@ class TestGenericReactionParameterBlock(object):
                            "argument."):
             m.rxn_params = GenericReactionParameterBlock(default={
                 "property_package": m.params,
+                "base_units": base_units,
                 "equilibrium_reactions": {
                     "e1": {"stoichiometry": {("p2", "c1"): -3,
                                              ("p2", "c2"): 4},
@@ -249,6 +324,7 @@ class TestGenericReactionParameterBlock(object):
     def test_rate_and_equil_build(self, m):
         m.rxn_params = GenericReactionParameterBlock(default={
             "property_package": m.params,
+            "base_units": base_units,
             "rate_reactions": {
                 "r1": {"stoichiometry": {("p1", "c1"): -1,
                                          ("p1", "c2"): 2},
@@ -310,6 +386,7 @@ class TestGenericReactionParameterBlock(object):
     def test_build_parameters(self, m):
         m.rxn_params = GenericReactionParameterBlock(default={
             "property_package": m.params,
+            "base_units": base_units,
             "rate_reactions": {
                 "r1": {"stoichiometry": {("p1", "c1"): -1,
                                          ("p1", "c2"): 2},
@@ -344,22 +421,25 @@ class TestGenericReactionBlock(object):
         m.params = GenericParameterBlock(default={
                 "components": {"c1": {}, "c2": {}},
                 "phases": {
-                    "p1": {"equation_of_state": dummy_eos},
-                    "p2": {"equation_of_state": dummy_eos}},
+                    "p1": {"equation_of_state": DummyEoS},
+                    "p2": {"equation_of_state": DummyEoS}},
                 "state_definition": modules[__name__],
                 "pressure_ref": 1e5,
-                "temperature_ref": 300})
+                "temperature_ref": 300,
+                "base_units": base_units})
 
         m.sblock = m.params.build_state_block([1])
 
         m.rxn_params = GenericReactionParameterBlock(default={
             "property_package": m.params,
+            "base_units": base_units,
             "rate_reactions": {
                 "r1": {"stoichiometry": {("p1", "c1"): -1,
                                          ("p1", "c2"): 2},
                        "heat_of_reaction": constant_dh_rxn,
                        "rate_constant": arrhenius,
-                       "rate_form": mole_frac_power_law_rate,
+                       "rate_form": power_law_rate,
+                       "concentration_form": ConcentrationForm.moleFraction,
                        "parameter_data": {
                            "dh_rxn_ref": -10000,
                            "arrhenius_const": 1,
@@ -369,7 +449,8 @@ class TestGenericReactionBlock(object):
                                          ("p2", "c2"): 4},
                        "heat_of_reaction": constant_dh_rxn,
                        "equilibrium_constant": van_t_hoff,
-                       "equilibrium_form": mole_frac_power_law_equil,
+                       "equilibrium_form": power_law_equil,
+                       "concentration_form": ConcentrationForm.moleFraction,
                        "parameter_data": {
                            "dh_rxn_ref": -20000,
                            "k_eq_ref": 100,
@@ -381,6 +462,13 @@ class TestGenericReactionBlock(object):
 
         return m
 
+    @pytest.mark.unit
+    def test_component_phase_lists(self, model):
+        assert model.rblock[1].component_list is model.params.component_list
+        assert model.rblock[1].phase_list is model.params.phase_list
+        assert model.rblock[1].phase_component_set is \
+            model.params._phase_component_set
+
     @pytest.mark.unit
     def test_dh_rxn(self, model):
         assert isinstance(model.rxn_params.reaction_r1.dh_rxn_ref, Var)
@@ -421,6 +509,16 @@ class TestGenericReactionBlock(object):
             model.rblock[1].k_rxn["r1"] *
             model.sblock[1].mole_frac_phase_comp["p1", "c1"]**1)
 
+    @pytest.mark.unit
+    def test_reaction_rate_None(self, model):
+        model.rxn_params.config.rate_reactions.r1.rate_form = None
+
+        with pytest.raises(ConfigurationError,
+                           match="rblock\[1\] Generic Reaction r1 was not "
+                           "provided with a rate_form configuration "
+                           "argument."):
+            model.rblock[1].reaction_rate
+
     @pytest.mark.unit
     def test_equilibrium_constant(self, model):
         assert isinstance(model.rxn_params.reaction_e1.k_eq_ref, Var)
@@ -450,3 +548,25 @@ class TestGenericReactionBlock(object):
             value(model.rblock[1].k_eq["e1"] -
                   model.sblock[1].mole_frac_phase_comp["p2", "c1"]**-3 *
                   model.sblock[1].mole_frac_phase_comp["p2", "c2"]**4)
+
+    @pytest.mark.unit
+    def test_basic_scaling(self, model):
+        model.rblock[1].calculate_scaling_factors()
+
+        assert len(model.rblock[1].scaling_factor) == 3
+
+        assert model.rblock[1].scaling_factor[
+            model.rblock[1].dh_rxn["e1"]] == 5e-5
+        assert model.rblock[1].scaling_factor[
+            model.rblock[1].dh_rxn["r1"]] == 1e-4
+        assert model.rblock[1].scaling_factor[
+            model.rblock[1].k_eq["e1"]] == 1e-2
+
+        # Check that scaling factor was added to equilibrium constraint
+        assert str(model.rblock[1].equilibrium_constraint["e1"].body) == \
+            str((model.rblock[1].k_eq["e1"] -
+                 model.sblock[1].mole_frac_phase_comp["p2", "c1"] **
+                 model.rxn_params.reaction_e1.reaction_order["p2", "c1"] *
+                 model.sblock[1].mole_frac_phase_comp["p2", "c2"] **
+                 model.rxn_params.reaction_e1.reaction_order["p2", "c2"]) *
+                0.01)
