@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 import pytest
 
 from idaes.core import FlowsheetBlock
@@ -19,12 +19,13 @@ from idaes.generic_models.properties.cubic_eos import BT_PR
 
 from pyomo.environ import (ConcreteModel,
                            Objective,
-                           SolverFactory,
                            TerminationCondition,
                            value)
+from pyomo.util.check_units import assert_units_consistent
 
-from idaes.generic_models.properties.tests.test_harness import PropertyTestHarness
-from idaes.core.util.testing import get_default_solver
+from idaes.generic_models.properties.tests.test_harness import \
+    PropertyTestHarness
+from idaes.core.util import get_solver
 
 
 # Set module level pyest marker
@@ -34,7 +35,7 @@ prop_available = cubic_roots_available()
 
 # -----------------------------------------------------------------------------
 # Get default solver for testing
-solver = get_default_solver()
+solver = get_solver()
 
 
 # -----------------------------------------------------------------------------
@@ -73,6 +74,20 @@ class TestBasicV(PropertyTestHarness):
 @pytest.mark.skipif(not prop_available,
                     reason="Cubic root finder not available")
 class TestBTExample(object):
+    @pytest.mark.component
+    def test_units(self):
+        m = ConcreteModel()
+
+        m.fs = FlowsheetBlock(default={'dynamic': False})
+
+        m.fs.props = BT_PR.BTParameterBlock(
+                default={'valid_phase': ('Vap', 'Liq')})
+
+        m.fs.state = m.fs.props.build_state_block(
+            default={"defined_state": True})
+
+        assert_units_consistent(m)
+
     @pytest.mark.integration
     def test_T_sweep(self):
         m = ConcreteModel()
@@ -96,12 +111,11 @@ class TestBTExample(object):
             m.fs.state.temperature.fix(300)
             m.fs.state.pressure.fix(10**(0.5*logP))
 
-            m.fs.state.initialize(outlvl=0)
+            m.fs.state.initialize()
 
             m.fs.state.temperature.unfix()
             m.fs.obj.activate()
 
-            solver = SolverFactory('ipopt')
             results = solver.solve(m, tee=True)
 
             assert results.solver.termination_condition == \
@@ -127,9 +141,8 @@ class TestBTExample(object):
             m.fs.state.temperature.fix(T)
             m.fs.state.pressure.fix(1e5)
 
-            m.fs.state.initialize(outlvl=0)
+            m.fs.state.initialize()
 
-            solver = SolverFactory('ipopt')
             results = solver.solve(m)
 
             assert results.solver.termination_condition == \
@@ -137,7 +150,7 @@ class TestBTExample(object):
 
             while m.fs.state.pressure.value <= 1e6:
                 m.fs.state.pressure.value = m.fs.state.pressure.value + 1e5
-                solver = SolverFactory('ipopt')
+
                 results = solver.solve(m)
                 assert results.solver.termination_condition == \
                     TerminationCondition.optimal
@@ -165,9 +178,8 @@ class TestBTExample(object):
         m.fs.state.enth_mol_phase
         m.fs.state.entr_mol_phase
 
-        m.fs.state.initialize(outlvl=0)
+        m.fs.state.initialize()
 
-        solver = SolverFactory('ipopt')
         solver.solve(m)
 
         assert pytest.approx(value(m.fs.state._teq), abs=1e-1) == 365
@@ -232,9 +244,8 @@ class TestBTExample(object):
         m.fs.state.enth_mol_phase
         m.fs.state.entr_mol_phase
 
-        m.fs.state.initialize(outlvl=0)
+        m.fs.state.initialize()
 
-        solver = SolverFactory('ipopt')
         solver.solve(m)
 
         assert pytest.approx(value(m.fs.state._teq), 1e-5) == 431.47
@@ -299,9 +310,8 @@ class TestBTExample(object):
         m.fs.state.enth_mol_phase
         m.fs.state.entr_mol_phase
 
-        m.fs.state.initialize(outlvl=0)
+        m.fs.state.initialize()
 
-        solver = SolverFactory('ipopt')
         solver.solve(m)
 
         assert pytest.approx(value(m.fs.state._teq), 1e-5) == 371.4
@@ -366,9 +376,8 @@ class TestBTExample(object):
         m.fs.state.enth_mol_phase
         m.fs.state.entr_mol_phase
 
-        m.fs.state.initialize(outlvl=0)
+        m.fs.state.initialize()
 
-        solver = SolverFactory('ipopt')
         solver.solve(m)
 
         assert pytest.approx(value(m.fs.state._teq), 1e-5) == 436.93
@@ -433,9 +442,8 @@ class TestBTExample(object):
         m.fs.state.enth_mol_phase
         m.fs.state.entr_mol_phase
 
-        m.fs.state.initialize(outlvl=0)
+        m.fs.state.initialize()
 
-        solver = SolverFactory('ipopt')
         solver.solve(m)
 
         assert pytest.approx(value(m.fs.state._teq), 1e-5) == 368
@@ -500,9 +508,8 @@ class TestBTExample(object):
         m.fs.state.enth_mol_phase
         m.fs.state.entr_mol_phase
 
-        m.fs.state.initialize(outlvl=0)
+        m.fs.state.initialize()
 
-        solver = SolverFactory('ipopt')
         solver.solve(m)
 
         assert pytest.approx(value(m.fs.state._teq), 1e-5) == 376
