@@ -1,32 +1,30 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 Basic IDAES 1D Heat Exchanger Model.
 
 1D Single pass shell and tube HX model with 0D wall conduction model
 """
 # Import Python libraries
-import math
 from enum import Enum
 
 # Import Pyomo libraries
 from pyomo.environ import (
     SolverFactory,
     Var,
-    Param,
     Constraint,
     value,
-    TerminationCondition,
+    units as pyunits
 )
 from pyomo.common.config import ConfigBlock, ConfigValue, In
 
@@ -41,11 +39,15 @@ from idaes.core import (
     FlowDirection,
     useDefault,
 )
-from idaes.generic_models.unit_models.heat_exchanger import HeatExchangerFlowPattern
+from idaes.generic_models.unit_models.heat_exchanger \
+    import HeatExchangerFlowPattern
 from idaes.core.util.config import is_physical_parameter_block
 from idaes.core.util.misc import add_object_reference
 from idaes.core.util.exceptions import ConfigurationError
 from idaes.core.util.tables import create_stream_table_dataframe
+from idaes.core.util.constants import Constants as c
+from idaes.core.util import get_solver, scaling as iscale
+
 import idaes.logger as idaeslog
 
 
@@ -139,7 +141,8 @@ balance type
             default=MomentumBalanceType.pressureTotal,
             domain=In(MomentumBalanceType),
             description="Momentum balance construction flag",
-            doc="""Indicates what type of momentum balance should be constructed,
+            doc="""Indicates what type of momentum balance should
+be constructed,
 **default** - MomentumBalanceType.pressureTotal.
 **Valid values:** {
 **MomentumBalanceType.none** - exclude momentum balances,
@@ -180,7 +183,8 @@ constructed,
             default=None,
             domain=is_physical_parameter_block,
             description="Property package to use for control volume",
-            doc="""Property parameter object used to define property calculations
+            doc="""Property parameter object used to define property
+calculations
 (default = 'use_parent_value')
 - 'use_parent_value' - get package from parent (default = None)
 - a ParameterBlock object""",
@@ -205,8 +209,8 @@ and used when constructing these
         ConfigValue(
             default=useDefault,
             description="Discretization method to use for DAE transformation",
-            doc="""Discretization method to use for DAE transformation. See Pyomo
-documentation for supported transformations.""",
+            doc="""Discretization method to use for DAE transformation. See
+Pyomo documentation for supported transformations.""",
         ),
     )
     _SideTemplate.declare(
@@ -220,8 +224,10 @@ Pyomo documentation for supported schemes.""",
     )
 
     # Create individual config blocks for shell and tube side
-    CONFIG.declare("shell_side", _SideTemplate(doc="shell side config arguments"))
-    CONFIG.declare("tube_side", _SideTemplate(doc="tube side config arguments"))
+    CONFIG.declare(
+        "shell_side", _SideTemplate(doc="shell side config arguments"))
+    CONFIG.declare(
+        "tube_side", _SideTemplate(doc="tube side config arguments"))
 
     # Common config args for both sides
     CONFIG.declare(
@@ -310,7 +316,8 @@ thickness of the tube""",
                     "Defaulting to finite "
                     "difference method on the shell side."
                 )
-                self.config.shell_side.transformation_method = "dae.finite_difference"
+                self.config.shell_side.transformation_method = \
+                    "dae.finite_difference"
             if self.config.tube_side.transformation_method is useDefault:
                 _log.warning(
                     "Discretization method was "
@@ -319,7 +326,8 @@ thickness of the tube""",
                     "Defaulting to finite "
                     "difference method on the tube side."
                 )
-                self.config.tube_side.transformation_method = "dae.finite_difference"
+                self.config.tube_side.transformation_method = \
+                    "dae.finite_difference"
             if self.config.shell_side.transformation_scheme is useDefault:
                 _log.warning(
                     "Discretization scheme was "
@@ -349,7 +357,8 @@ thickness of the tube""",
                     "Defaulting to finite "
                     "difference method on the shell side."
                 )
-                self.config.shell_side.transformation_method = "dae.finite_difference"
+                self.config.shell_side.transformation_method = \
+                    "dae.finite_difference"
             if self.config.tube_side.transformation_method is useDefault:
                 _log.warning(
                     "Discretization method was "
@@ -358,7 +367,8 @@ thickness of the tube""",
                     "Defaulting to finite "
                     "difference method on the tube side."
                 )
-                self.config.tube_side.transformation_method = "dae.finite_difference"
+                self.config.tube_side.transformation_method = \
+                    "dae.finite_difference"
             if self.config.shell_side.transformation_scheme is useDefault:
                 _log.warning(
                     "Discretization scheme was "
@@ -381,7 +391,8 @@ thickness of the tube""",
             raise ConfigurationError(
                 "{} HeatExchanger1D only supports cocurrent and "
                 "countercurrent flow patterns, but flow_type configuration"
-                " argument was set to {}.".format(self.name, self.config.flow_type)
+                " argument was set to {}."
+                .format(self.name, self.config.flow_type)
             )
 
         # Control volume 1D for shell
@@ -390,9 +401,12 @@ thickness of the tube""",
                 "dynamic": self.config.shell_side.dynamic,
                 "has_holdup": self.config.shell_side.has_holdup,
                 "property_package": self.config.shell_side.property_package,
-                "property_package_args": self.config.shell_side.property_package_args,
-                "transformation_method": self.config.shell_side.transformation_method,
-                "transformation_scheme": self.config.shell_side.transformation_scheme,
+                "property_package_args":
+                self.config.shell_side.property_package_args,
+                "transformation_method":
+                self.config.shell_side.transformation_method,
+                "transformation_scheme":
+                self.config.shell_side.transformation_scheme,
                 "finite_elements": self.config.finite_elements,
                 "collocation_points": self.config.collocation_points,
             }
@@ -403,9 +417,12 @@ thickness of the tube""",
                 "dynamic": self.config.tube_side.dynamic,
                 "has_holdup": self.config.tube_side.has_holdup,
                 "property_package": self.config.tube_side.property_package,
-                "property_package_args": self.config.tube_side.property_package_args,
-                "transformation_method": self.config.tube_side.transformation_method,
-                "transformation_scheme": self.config.tube_side.transformation_scheme,
+                "property_package_args":
+                self.config.tube_side.property_package_args,
+                "transformation_method":
+                self.config.tube_side.transformation_method,
+                "transformation_scheme":
+                self.config.tube_side.transformation_scheme,
                 "finite_elements": self.config.finite_elements,
                 "collocation_points": self.config.collocation_points,
             }
@@ -485,51 +502,67 @@ thickness of the tube""",
         Returns:
             None
         """
-        # Unit model parameters
-        self.pi = Param(initialize=math.pi, doc="pi")
+        shell_units = self.config.shell_side.property_package.\
+            get_metadata().get_derived_units
+        tube_units = self.config.tube_side.property_package.\
+            get_metadata().get_derived_units
 
         # Unit model variables
         # HX dimensions
-        self.d_shell = Var(initialize=1, doc="Diameter of shell")
-        self.d_tube_outer = Var(initialize=0.011, doc="Outer diameter of tube")
-        self.d_tube_inner = Var(initialize=0.010, doc="Inner diameter of tube")
-        self.N_tubes = Var(initialize=1, doc="Number of tubes")
+        self.d_shell = Var(initialize=1,
+                           doc="Diameter of shell",
+                           units=shell_units("length"))
+        self.d_tube_outer = Var(initialize=0.011,
+                                doc="Outer diameter of tube",
+                           units=shell_units("length"))
+        self.d_tube_inner = Var(initialize=0.010,
+                                doc="Inner diameter of tube",
+                           units=shell_units("length"))
+        self.N_tubes = Var(initialize=1,
+                           doc="Number of tubes",
+                           units=pyunits.dimensionless)
 
         # Note: In addition to the above variables, "shell_length" and
         # "tube_length" need to be fixed at the flowsheet level
 
         # Performance variables
         self.shell_heat_transfer_coefficient = Var(
-            self.flowsheet().config.time,
+            self.flowsheet().time,
             self.shell.length_domain,
             initialize=50,
             doc="Heat transfer coefficient",
+            units=shell_units("heat_transfer_coefficient")
         )
         self.tube_heat_transfer_coefficient = Var(
-            self.flowsheet().config.time,
+            self.flowsheet().time,
             self.tube.length_domain,
             initialize=50,
-            doc="Heat transfer " "coefficient",
+            doc="Heat transfer coefficient",
+            units=tube_units("heat_transfer_coefficient")
         )
 
         # Wall 0D model (Q_shell = Q_tube*N_tubes)
-        if self.config.has_wall_conduction == WallConductionType.zero_dimensional:
+        if self.config.has_wall_conduction == \
+                WallConductionType.zero_dimensional:
             self.temperature_wall = Var(
-                self.flowsheet().config.time, self.tube.length_domain, initialize=298.15
+                self.flowsheet().time,
+                self.tube.length_domain,
+                initialize=298.15,
+                units=shell_units("temperature")
             )
 
             # Performance equations
             # Energy transfer between shell and tube wall
 
             @self.Constraint(
-                self.flowsheet().config.time,
+                self.flowsheet().time,
                 self.shell.length_domain,
                 doc="Heat transfer between shell and tube",
             )
             def shell_heat_transfer_eq(self, t, x):
                 return self.shell.heat[t, x] == -self.N_tubes * (
                     self.shell_heat_transfer_coefficient[t, x]
-                    * self.pi
+                    * c.pi
                     * self.d_tube_outer
                     * (
                         self.shell.properties[t, x].temperature
@@ -539,25 +572,36 @@ thickness of the tube""",
 
             # Energy transfer between tube wall and tube
             @self.Constraint(
-                self.flowsheet().config.time,
+                self.flowsheet().time,
                 self.tube.length_domain,
                 doc="Convective heat transfer",
             )
             def tube_heat_transfer_eq(self, t, x):
-                return self.tube.heat[t, x] == self.tube_heat_transfer_coefficient[
+                return self.tube.heat[t, x] == \
+                    self.tube_heat_transfer_coefficient[
                     t, x
-                ] * self.pi * self.d_tube_inner * (
-                    self.temperature_wall[t, x] - self.tube.properties[t, x].temperature
+                ] * c.pi * pyunits.convert(self.d_tube_inner,
+                                           to_units=tube_units("length")) * (
+                    pyunits.convert(self.temperature_wall[t, x],
+                                    to_units=tube_units('temperature')) -
+                    self.tube.properties[t, x].temperature
                 )
 
+            if shell_units("length") is None:
+                # Backwards compatability check
+                q_units = None
+            else:
+                q_units = shell_units("power")/shell_units("length")
             # Wall 0D model
             @self.Constraint(
-                self.flowsheet().config.time,
+                self.flowsheet().time,
                 self.shell.length_domain,
                 doc="wall 0D model",
             )
             def wall_0D_model(self, t, x):
-                return self.tube.heat[t, x] == -(self.shell.heat[t, x] / self.N_tubes)
+                return pyunits.convert(self.tube.heat[t, x],
+                                       to_units=q_units) == -(
+                            self.shell.heat[t, x] / self.N_tubes)
 
         else:
             raise NotImplementedError(
@@ -567,25 +611,26 @@ thickness of the tube""",
 
         # Define tube area in terms of tube diameter
         self.area_calc_tube = Constraint(
-            expr=4 * self.tube_area == self.pi * self.d_tube_inner ** 2
+            expr=4 * self.tube_area == c.pi * pyunits.convert(
+                self.d_tube_inner, to_units=tube_units("length"))**2
         )
 
         # Define shell area in terms of shell and tube diameter
         self.area_calc_shell = Constraint(
             expr=4 * self.shell_area
-            == self.pi * (self.d_shell ** 2 - self.N_tubes * self.d_tube_outer ** 2)
+            == c.pi * (self.d_shell**2 - self.N_tubes*self.d_tube_outer**2)
         )
 
     def initialize(
-        blk,
+        self,
         shell_state_args=None,
         tube_state_args=None,
         outlvl=idaeslog.NOTSET,
-        solver="ipopt",
-        optarg={"tol": 1e-6},
+        solver=None,
+        optarg=None,
     ):
         """
-        Initialization routine for the unit (default solver ipopt).
+        Initialization routine for the unit.
 
         Keyword Arguments:
             state_args : a dict of arguments to be passed to the property
@@ -593,29 +638,30 @@ thickness of the tube""",
                          initialization (see documentation of the specific
                          property package) (default = {}).
             outlvl : sets output level of initialization routine
-            optarg : solver options dictionary object (default={'tol': 1e-6})
-            solver : str indicating whcih solver to use during
-                     initialization (default = 'ipopt')
+            optarg : solver options dictionary object (default=None, use
+                     default solver options)
+            solver : str indicating which solver to use during
+                     initialization (default = None, use default solver)
 
         Returns:
             None
         """
-        init_log = idaeslog.getInitLogger(blk.name, outlvl, tag="unit")
-        solve_log = idaeslog.getSolveLogger(blk.name, outlvl, tag="unit")
+        init_log = idaeslog.getInitLogger(self.name, outlvl, tag="unit")
+        solve_log = idaeslog.getSolveLogger(self.name, outlvl, tag="unit")
 
-        opt = SolverFactory(solver)
-        opt.options = optarg
+        # Create solver
+        opt = get_solver(solver, optarg)
 
         # ---------------------------------------------------------------------
         # Initialize shell block
-        flags_shell = blk.shell.initialize(
+        flags_shell = self.shell.initialize(
             outlvl=outlvl,
             optarg=optarg,
             solver=solver,
             state_args=shell_state_args,
         )
 
-        flags_tube = blk.tube.initialize(
+        flags_tube = self.tube.initialize(
             outlvl=outlvl,
             optarg=optarg,
             solver=solver,
@@ -627,49 +673,54 @@ thickness of the tube""",
         # ---------------------------------------------------------------------
         # Solve unit
         # Wall 0D
-        if blk.config.has_wall_conduction == WallConductionType.zero_dimensional:
-            for t in blk.flowsheet().config.time:
-                for z in blk.shell.length_domain:
-                    blk.temperature_wall[t, z].fix(
+        if self.config.has_wall_conduction == \
+            WallConductionType.zero_dimensional:
+            shell_units = self.config.shell_side.property_package.\
+                get_metadata().get_derived_units
+            for t in self.flowsheet().time:
+                for z in self.shell.length_domain:
+                    self.temperature_wall[t, z].fix(
                         value(
                             0.5
                             * (
-                                blk.shell.properties[t, 0].temperature
-                                + blk.tube.properties[t, 0].temperature
+                                self.shell.properties[t, 0].temperature
+                                + pyunits.convert(
+                                    self.tube.properties[t, 0].temperature,
+                                    to_units=shell_units('temperature'))
                             )
                         )
                     )
 
-            blk.tube.deactivate()
-            blk.tube_heat_transfer_eq.deactivate()
-            blk.wall_0D_model.deactivate()
+            self.tube.deactivate()
+            self.tube_heat_transfer_eq.deactivate()
+            self.wall_0D_model.deactivate()
 
             with idaeslog.solver_log(solve_log, idaeslog.DEBUG) as slc:
-                res = opt.solve(blk, tee=slc.tee)
+                res = opt.solve(self, tee=slc.tee)
             init_log.info_high(
                 "Initialization Step 2 {}.".format(idaeslog.condition(res))
             )
 
-            blk.tube.activate()
-            blk.tube_heat_transfer_eq.activate()
+            self.tube.activate()
+            self.tube_heat_transfer_eq.activate()
 
             with idaeslog.solver_log(solve_log, idaeslog.DEBUG) as slc:
-                res = opt.solve(blk, tee=slc.tee)
+                res = opt.solve(self, tee=slc.tee)
             init_log.info_high(
                 "Initialization Step 3 {}.".format(idaeslog.condition(res))
             )
 
-            blk.wall_0D_model.activate()
-            blk.temperature_wall.unfix()
+            self.wall_0D_model.activate()
+            self.temperature_wall.unfix()
 
             with idaeslog.solver_log(solve_log, idaeslog.DEBUG) as slc:
-                res = opt.solve(blk, tee=slc.tee)
+                res = opt.solve(self, tee=slc.tee)
             init_log.info_high(
                 "Initialization Step 4 {}.".format(idaeslog.condition(res))
             )
 
-        blk.shell.release_state(flags_shell)
-        blk.tube.release_state(flags_tube)
+        self.shell.release_state(flags_shell)
+        self.tube.release_state(flags_tube)
 
         init_log.info("Initialization Complete.")
 
@@ -696,3 +747,17 @@ thickness of the tube""",
             },
             time_point=time_point,
         )
+
+    def calculate_scaling_factors(self):
+        super().calculate_scaling_factors()
+
+        for i, c in self.shell_heat_transfer_eq.items():
+            iscale.constraint_scaling_transform(
+                c, iscale.get_scaling_factor(
+                    self.shell.heat[i], default=1, warning=True),
+                overwrite=False)
+
+        for i, c in self.tube_heat_transfer_eq.items():
+            iscale.constraint_scaling_transform(c, iscale.get_scaling_factor(
+                    self.tube.heat[i], default=1, warning=True),
+                overwrite=False)
