@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 General purpose mixer block for IDAES models
 """
@@ -31,6 +31,7 @@ from idaes.core import (
     UnitModelBlockData,
     useDefault,
     MaterialBalanceType,
+    MaterialFlowBasis
 )
 from idaes.core.util.config import (
     is_physical_parameter_block,
@@ -44,6 +45,9 @@ from idaes.core.util.exceptions import (
 )
 from idaes.core.util.math import smooth_min
 from idaes.core.util.tables import create_stream_table_dataframe
+import idaes.core.util.scaling as iscale
+from idaes.core.util import get_solver
+
 import idaes.logger as idaeslog
 
 __author__ = "Andrew Lee"
@@ -291,7 +295,7 @@ objects linked to all inlet states and the mixed state,
 
         mb_type = self.config.material_balance_type
         if mb_type == MaterialBalanceType.useDefault:
-            t_ref = self.flowsheet().config.time.first()
+            t_ref = self.flowsheet().time.first()
             mb_type = mixed_block[t_ref].default_material_balance_type()
 
         if mb_type != MaterialBalanceType.none:
@@ -418,7 +422,7 @@ objects linked to all inlet states and the mixed state,
         # Create an instance of StateBlock for all inlets
         for i in inlet_list:
             i_obj = self.config.property_package.build_state_block(
-                self.flowsheet().config.time,
+                self.flowsheet().time,
                 doc="Material properties at inlet",
                 default=tmp_dict,
             )
@@ -442,7 +446,7 @@ objects linked to all inlet states and the mixed state,
         tmp_dict["defined_state"] = False
 
         self.mixed_state = self.config.property_package.build_state_block(
-            self.flowsheet().config.time,
+            self.flowsheet().time,
             doc="Material properties of mixed stream",
             default=tmp_dict,
         )
@@ -467,7 +471,7 @@ objects linked to all inlet states and the mixed state,
         # Check that the user-provided StateBlock uses the same prop pack
         if (
             self.config.mixed_state_block[
-                self.flowsheet().config.time.first()
+                self.flowsheet().time.first()
             ].config.parameters
             != self.config.property_package
         ):
@@ -488,27 +492,31 @@ objects linked to all inlet states and the mixed state,
         """
         pp = self.config.property_package
         # Get phase component list(s)
-        pc_set = pp.get_phase_component_set()
+        pc_set = mixed_block.phase_component_set
+
+        # Get units metadata
+        units = pp.get_metadata()
+
+        flow_basis = mixed_block[
+            self.flowsheet().time.first()].get_material_flow_basis()
+        if flow_basis == MaterialFlowBasis.molar:
+            flow_units = units.get_derived_units("flow_mole")
+        elif flow_basis == MaterialFlowBasis.mass:
+            flow_units = units.get_derived_units("flow_mass")
+        else:
+            # Let this pass for now with no units
+            flow_units = None
 
         if mb_type == MaterialBalanceType.componentPhase:
             # Create equilibrium generation term and constraints if required
             if self.config.has_phase_equilibrium is True:
-                # Get units from property package
-                units = {}
-                for u in ["holdup", "time"]:
-                    try:
-                        units[u] = pp.get_metadata().default_units[u]
-                    except KeyError:
-                        units[u] = "-"
-
                 try:
                     self.phase_equilibrium_generation = Var(
-                        self.flowsheet().config.time,
+                        self.flowsheet().time,
                         pp.phase_equilibrium_idx,
                         domain=Reals,
-                        doc="Amount of generation in unit by phase "
-                        "equilibria [{}/{}]".format(units["holdup"],
-                                                    units["time"]),
+                        doc="Amount of generation in unit by phase equilibria",
+                        units=flow_units
                     )
                 except AttributeError:
                     raise PropertyNotSupportedError(
@@ -552,28 +560,25 @@ objects linked to all inlet states and the mixed state,
 
             # Write phase-component balances
             @self.Constraint(
-                self.flowsheet().config.time,
+                self.flowsheet().time,
                 pc_set,
                 doc="Material mixing equations",
             )
             def material_mixing_equations(b, t, p, j):
-                if (p, j) in pc_set:
-                    return 0 == (
-                        sum(
-                            inlet_blocks[i][t].get_material_flow_terms(p, j)
-                            for i in range(len(inlet_blocks))
-                        )
-                        - mixed_block[t].get_material_flow_terms(p, j)
-                        + phase_equilibrium_term(b, t, p, j)
+                return 0 == (
+                    sum(
+                        inlet_blocks[i][t].get_material_flow_terms(p, j)
+                        for i in range(len(inlet_blocks))
                     )
-                else:
-                    return Constraint.Skip
+                    - mixed_block[t].get_material_flow_terms(p, j)
+                    + phase_equilibrium_term(b, t, p, j)
+                )
 
         elif mb_type == MaterialBalanceType.componentTotal:
             # Write phase-component balances
             @self.Constraint(
-                self.flowsheet().config.time,
-                self.config.property_package.component_list,
+                self.flowsheet().time,
+                mixed_block.component_list,
                 doc="Material mixing equations",
             )
             def material_mixing_equations(b, t, j):
@@ -583,14 +588,14 @@ objects linked to all inlet states and the mixed state,
                         for i in range(len(inlet_blocks))
                     )
                     - mixed_block[t].get_material_flow_terms(p, j)
-                    for p in b.config.property_package.phase_list
+                    for p in mixed_block.phase_list
                     if (p, j) in pc_set
                 )
 
         elif mb_type == MaterialBalanceType.total:
             # Write phase-component balances
             @self.Constraint(
-                self.flowsheet().config.time, doc="Material mixing equations"
+                self.flowsheet().time, doc="Material mixing equations"
             )
             def material_mixing_equations(b, t):
                 return 0 == sum(
@@ -600,10 +605,10 @@ objects linked to all inlet states and the mixed state,
                             for i in range(len(inlet_blocks))
                         )
                         - mixed_block[t].get_material_flow_terms(p, j)
-                        for j in b.config.property_package.component_list
+                        for j in mixed_block.component_list
                         if (p, j) in pc_set
                     )
-                    for p in b.config.property_package.phase_list
+                    for p in mixed_block.phase_list
                 )
 
         elif mb_type == MaterialBalanceType.elementTotal:
@@ -625,23 +630,20 @@ objects linked to all inlet states and the mixed state,
         """
         Add energy mixing equations (total enthalpy balance).
         """
-        self.scaling_factor_energy = Param(
-            default=1e-6, mutable=True, doc="Energy balance scaling parameter"
-        )
 
-        @self.Constraint(self.flowsheet().config.time, doc="Energy balances")
+        @self.Constraint(self.flowsheet().time, doc="Energy balances")
         def enthalpy_mixing_equations(b, t):
-            return 0 == self.scaling_factor_energy * (
+            return 0 == (
                 sum(
                     sum(
                         inlet_blocks[i][t].get_enthalpy_flow_terms(p)
-                        for p in b.config.property_package.phase_list
+                        for p in mixed_block.phase_list
                     )
                     for i in range(len(inlet_blocks))
                 )
                 - sum(
                     mixed_block[t].get_enthalpy_flow_terms(p)
-                    for p in b.config.property_package.phase_list
+                    for p in mixed_block.phase_list
                 )
             )
 
@@ -653,23 +655,29 @@ objects linked to all inlet states and the mixed state,
         """
         if not hasattr(self, "inlet_idx"):
             self.inlet_idx = RangeSet(len(inlet_blocks))
+
+        # Get units metadata
+        units = self.config.property_package.get_metadata()
+
         # Add variables
         self.minimum_pressure = Var(
-            self.flowsheet().config.time,
+            self.flowsheet().time,
             self.inlet_idx,
-            doc="Variable for calculating " "minimum inlet pressure",
+            doc="Variable for calculating minimum inlet pressure",
+            units=units.get_derived_units("pressure")
         )
 
         self.eps_pressure = Param(
             mutable=True,
             initialize=1e-3,
             domain=PositiveReals,
-            doc="Smoothing term for " "minimum inlet pressure",
+            doc="Smoothing term for minimum inlet pressure",
+            units=units.get_derived_units("pressure")
         )
 
         # Calculate minimum inlet pressure
         @self.Constraint(
-            self.flowsheet().config.time,
+            self.flowsheet().time,
             self.inlet_idx,
             doc="Calculation for minimum inlet pressure",
         )
@@ -688,7 +696,7 @@ objects linked to all inlet states and the mixed state,
 
         # Set inlet pressure to minimum pressure
         @self.Constraint(
-            self.flowsheet().config.time, doc="Link pressure to control volume"
+            self.flowsheet().time, doc="Link pressure to control volume"
         )
         def mixture_pressure(b, t):
             return mixed_block[t].pressure == (
@@ -706,7 +714,7 @@ objects linked to all inlet states and the mixed state,
 
         # Create equality constraints
         @self.Constraint(
-            self.flowsheet().config.time,
+            self.flowsheet().time,
             self.inlet_idx,
             doc="Calculation for minimum inlet pressure",
         )
@@ -744,7 +752,7 @@ objects linked to all inlet states and the mixed state,
             None
         """
         # Try property block model check
-        for t in blk.flowsheet().config.time:
+        for t in blk.flowsheet().time:
             try:
                 inlet_list = blk.create_inlet_list()
                 for i in inlet_list:
@@ -804,15 +812,17 @@ objects linked to all inlet states and the mixed state,
         self.minimum_pressure_constraint.deactivate()
         self.pressure_equality_constraints.activate()
 
-    def initialize(blk, outlvl=6, optarg={}, solver="ipopt", hold_state=False):
+    def initialize(blk, outlvl=idaeslog.NOTSET, optarg=None,
+                   solver=None, hold_state=False):
         """
-        Initialization routine for mixer (default solver ipopt)
+        Initialization routine for mixer.
 
         Keyword Arguments:
             outlvl : sets output level of initialization routine
-            optarg : solver options dictionary object (default={})
-            solver : str indicating whcih solver to use during
-                     initialization (default = 'ipopt')
+            optarg : solver options dictionary object (default=None, use
+                     default solver options)
+            solver : str indicating which solver to use during
+                     initialization (default = None, use default solver)
             hold_state : flag indicating whether the initialization routine
                      should unfix any state variables fixed during
                      initialization, **default** - False. **Valid values:**
@@ -829,9 +839,8 @@ objects linked to all inlet states and the mixed state,
         init_log = idaeslog.getInitLogger(blk.name, outlvl, tag="unit")
         solve_log = idaeslog.getSolveLogger(blk.name, outlvl, tag="unit")
 
-        # Set solver options
-        opt = SolverFactory(solver)
-        opt.options = optarg
+        # Create solver
+        opt = get_solver(solver, optarg)
 
         # Initialize inlet state blocks
         flags = {}
@@ -856,7 +865,7 @@ objects linked to all inlet states and the mixed state,
 
         o_flags = {}
         # Calculate initial guesses for mixed stream state
-        for t in blk.flowsheet().config.time:
+        for t in blk.flowsheet().time:
             # Iterate over state vars as defined by property package
             s_vars = mblock[t].define_state_vars()
             for s in s_vars:
@@ -903,7 +912,7 @@ objects linked to all inlet states and the mixed state,
         )
 
         # Revert fixed status of variables to what they were before
-        for t in blk.flowsheet().config.time:
+        for t in blk.flowsheet().time:
             s_vars = mblock[t].define_state_vars()
             for s in s_vars:
                 for k in s_vars[s]:
@@ -915,7 +924,7 @@ objects linked to all inlet states and the mixed state,
                 and blk.pressure_equality_constraints.active is True
             ):
                 blk.pressure_equality_constraints.deactivate()
-                for t in blk.flowsheet().config.time:
+                for t in blk.flowsheet().time:
                     sys_press = getattr(
                         blk,
                         blk.create_inlet_list()[0] + "_state")[t].pressure
@@ -923,7 +932,7 @@ objects linked to all inlet states and the mixed state,
                 with idaeslog.solver_log(solve_log, idaeslog.DEBUG)as slc:
                     res = opt.solve(blk, tee=slc.tee)
                 blk.pressure_equality_constraints.activate()
-                for t in blk.flowsheet().config.time:
+                for t in blk.flowsheet().time:
                     blk.mixed_state[t].pressure.unfix()
             else:
                 with idaeslog.solver_log(solve_log, idaeslog.DEBUG) as slc:
@@ -957,7 +966,7 @@ objects linked to all inlet states and the mixed state,
         inlet_list = blk.create_inlet_list()
         for i in inlet_list:
             i_block = getattr(blk, i + "_state")
-            i_block.release_state(flags[i], outlvl=outlvl + 1)
+            i_block.release_state(flags[i], outlvl=outlvl)
 
     def _get_stream_table_contents(self, time_point=0):
         io_dict = {}
@@ -969,3 +978,58 @@ objects linked to all inlet states and the mixed state,
         else:
             io_dict["Outlet"] = self.config.mixed_state_block
         return create_stream_table_dataframe(io_dict, time_point=time_point)
+
+    def calculate_scaling_factors(self):
+        super().calculate_scaling_factors()
+        mb_type = self.config.material_balance_type
+        if mb_type == MaterialBalanceType.useDefault:
+            t_ref = self.flowsheet().time.first()
+            mb_type = self.mixed_state[t_ref].default_material_balance_type()
+
+        if hasattr(self, "pressure_equality_constraints"):
+            for (t, i), c in self.pressure_equality_constraints.items():
+                s = iscale.get_scaling_factor(
+                    self.mixed_state[t].pressure, default=1, warning=True)
+                iscale.constraint_scaling_transform(c, s)
+
+        if hasattr(self, "material_mixing_equations"):
+            if mb_type == MaterialBalanceType.componentPhase:
+                for (t, p, j), c in self.material_mixing_equations.items():
+                    flow_term = self.mixed_state[t].get_material_flow_terms(p, j)
+                    s = iscale.get_scaling_factor(flow_term, default=1)
+                    iscale.constraint_scaling_transform(c, s, overwrite=False)
+            elif mb_type == MaterialBalanceType.componentTotal:
+                for (t, j), c in self.material_mixing_equations.items():
+                    for i, p in enumerate(self.mixed_state.phase_list):
+                        try:
+                            ft = self.mixed_state[t].get_material_flow_terms(p, j)
+                        except (KeyError, AttributeError):
+                            continue # component not in phase
+                        if i == 0:
+                            s = iscale.get_scaling_factor(ft, default=1)
+                        else:
+                            _s = iscale.get_scaling_factor(ft, default=1)
+                            s = _s if _s < s else s
+                    iscale.constraint_scaling_transform(c, s, overwrite=False)
+            elif mb_type == MaterialBalanceType.total:
+                pc_set = self.mixed_state.phase_component_set
+                for t, c in self.material_mixing_equations.items():
+                    for i, (p, j) in enumerate(pc_set):
+                        ft = self.mixed_state[t].get_material_flow_terms(p, j)
+                        if i == 0:
+                            s = iscale.get_scaling_factor(ft, default=1)
+                        else:
+                            _s = iscale.get_scaling_factor(ft, default=1)
+                            s = _s if _s < s else s
+                    iscale.constraint_scaling_transform(c, s, overwrite=False)
+
+        if hasattr(self, "enthalpy_mixing_equations"):
+            for t, c in self.enthalpy_mixing_equations.items():
+                def scale_gen():
+                    for v in self.mixed_state[t].phase_list:
+                        yield self.mixed_state[t].get_enthalpy_flow_terms(p)
+
+                s = iscale.min_scaling_factor(
+                    scale_gen(),
+                    default=1)
+                iscale.constraint_scaling_transform(c, s, overwrite=False)
