@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 """
 Tests for ControlVolumeBlockData.
 
@@ -25,7 +25,10 @@ from pyomo.environ import (ConcreteModel,
                            Var,
                            TerminationCondition,
                            SolverStatus,
-                           value)
+                           value,
+                           units)
+from pyomo.util.check_units import (assert_units_consistent,
+                                    assert_units_equivalent)
 from pyomo.network import Port
 from pyomo.common.config import ConfigBlock
 
@@ -33,6 +36,7 @@ from idaes.core import (FlowsheetBlock,
                         StateBlock,
                         declare_process_block_class,
                         StateBlockData,
+                        StateBlock,
                         PhysicalParameterBlock,
                         MaterialBalanceType,
                         EnergyBalanceType)
@@ -52,18 +56,20 @@ from idaes.core.util.exceptions import (BurntToast,
 from idaes.core.util.model_statistics import (degrees_of_freedom,
                                               number_variables,
                                               number_total_constraints,
-                                              fixed_variables_set,
-                                              activated_constraints_set,
                                               number_unused_variables)
-from idaes.core.util.testing import (get_default_solver,
-                                     PhysicalParameterTestBlock,
+from idaes.core.util.testing import (PhysicalParameterTestBlock,
                                      TestStateBlock,
                                      initialization_tester)
+from idaes.generic_models.properties.core.generic.generic_property import (
+    GenericParameterBlock)
+from idaes.power_generation.properties.natural_gas_PR import get_prop
+import idaes.core.util.scaling as iscale
+from idaes.core.util import get_solver
 
 
 # -----------------------------------------------------------------------------
 # Get default solver for testing
-solver = get_default_solver()
+solver = get_solver()
 
 
 # -----------------------------------------------------------------------------
@@ -651,7 +657,7 @@ class TestMixer(object):
         assert m.fs.mix.mixed_state[0].check is True
 
     @pytest.mark.initialization
-    @pytest.mark.unit
+    @pytest.mark.component
     def test_initialize(self):
         m = ConcreteModel()
         m.fs = FlowsheetBlock(default={"dynamic": False})
@@ -690,7 +696,7 @@ class TestMixer(object):
         assert m.fs.sb[0].hold_state is False
 
     @pytest.mark.ui
-    @pytest.mark.unit
+    @pytest.mark.component
     def test_report(self):
         m = ConcreteModel()
         m.fs = FlowsheetBlock(default={"dynamic": False})
@@ -756,6 +762,10 @@ class TestBTX(object):
         assert number_total_constraints(btx) == 25
         assert number_unused_variables(btx) == 0
 
+    @pytest.mark.component
+    def test_units(self, btx):
+        assert_units_consistent(btx)
+
     @pytest.mark.unit
     def test_dof(self, btx):
         assert degrees_of_freedom(btx) == 0
@@ -838,7 +848,8 @@ class _NoPressureParameterBlock(PhysicalParameterBlock):
                                'holdup': 'mol'})
 
 
-@declare_process_block_class("NoPressureStateBlock")
+@declare_process_block_class("NoPressureStateBlock",
+                             block_class=StateBlock)
 class NoPressureStateBlockData(StateBlockData):
     CONFIG = ConfigBlock(implicit=True)
 
@@ -986,6 +997,10 @@ class TestIAPWS(object):
         assert hasattr(iapws.fs.unit.outlet, "enth_mol")
         assert hasattr(iapws.fs.unit.outlet, "pressure")
 
+    @pytest.mark.component
+    def test_units(self, iapws):
+        assert_units_consistent(iapws)
+
     @pytest.mark.unit
     def test_dof(self, iapws):
         assert degrees_of_freedom(iapws) == 0
@@ -1097,6 +1112,10 @@ class TestSaponification(object):
         assert number_total_constraints(sapon) == 10
         assert number_unused_variables(sapon) == 0
 
+    @pytest.mark.component
+    def test_units(self, sapon):
+        assert_units_consistent(sapon)
+
     @pytest.mark.unit
     def test_dof(self, sapon):
         assert degrees_of_freedom(sapon) == 0
@@ -1107,6 +1126,10 @@ class TestSaponification(object):
     def test_initialize(self, sapon):
         initialization_tester(sapon)
 
+    @pytest.mark.component
+    def test_scaling(self, sapon):
+        sapon.fs.unit.calculate_scaling_factors()
+
     @pytest.mark.skipif(solver is None, reason="Solver not available")
     @pytest.mark.component
     def test_solve(self, sapon):
@@ -1167,3 +1190,15 @@ class TestSaponification(object):
     @pytest.mark.unit
     def test_report(self, sapon):
         sapon.fs.unit.report()
+
+@pytest.mark.component
+def test_construction_component_not_in_phase():
+    m = ConcreteModel()
+    m.fs = FlowsheetBlock()
+    m.fs.prop_params = GenericParameterBlock(
+        default=get_prop(["H2O", "H2"], ["Liq", "Vap"]))
+    m.fs.inject1 = Mixer(default={
+        "property_package": m.fs.prop_params,
+        "inlet_list":["in1", "in2"],
+        "momentum_mixing_type":MomentumMixingType.none})
+    iscale.calculate_scaling_factors(m)
