@@ -1,15 +1,15 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 
 """This is an example supercritical pulverized coal (SCPC) power plant steam cycle
 model.  This model doesn't represent any specific power plant, but it represents
@@ -18,7 +18,7 @@ This model is for demonstration and tutorial purposes only. Before looking at th
 model, it may be useful to look at the process flow diagram (PFD).
 """
 
-__author__ = "John Eslick"
+__author__ = "John Eslick, Maojian Wang"
 
 # Import Python libraries
 from collections import OrderedDict
@@ -34,17 +34,19 @@ from idaes.core import FlowsheetBlock  # Flowsheet class
 from idaes.core.util import model_serializer as ms  # load/save model state
 from idaes.core.util.misc import svg_tag  # place numbers/text in an SVG
 from idaes.generic_models.properties import iapws95  # steam properties
-from idaes.power_generation.unit_models import (  # power generation unit models
-    TurbineMultistage,
-    FWH0D,
+from idaes.power_generation.unit_models.helm import (
+    HelmTurbineMultistage,
+    HelmMixer,
+    HelmIsentropicCompressor,
+    HelmTurbineStage,
+    HelmNtuCondenser as Condenser
 )
+from idaes.power_generation.unit_models import FWH0D
 from idaes.generic_models.unit_models import (  # basic IDAES unit models, and enum
-    Mixer,
     HeatExchanger,
-    PressureChanger,
     MomentumMixingType,  # Enum type for mixer pressure calculation selection
 )
-from idaes.core.util import copy_port_values as _set_port  # for model intialization
+from idaes.core.util import get_solver, copy_port_values as _set_port  # for model intialization
 from idaes.core.util.model_statistics import degrees_of_freedom
 from idaes.core.util.tables import create_stream_table_dataframe  # as Pandas DataFrame
 
@@ -54,6 +56,8 @@ from idaes.generic_models.unit_models.heat_exchanger import delta_temperature_un
 # Pressure changer type (e.g. adiabatic, pump, isentropic...)
 from idaes.generic_models.unit_models.pressure_changer import ThermodynamicAssumption
 import idaes.logger as idaeslog
+import idaes.core.util.scaling as iscale
+
 
 _log = idaeslog.getModelLogger(__name__, logging.INFO)
 
@@ -78,7 +82,7 @@ def create_model():
     # (PH) state variables.  Usually pressure and enthalpy state variables are
     # more robust especially when the phases are unknown.
     m.fs.prop_water = iapws95.Iapws95ParameterBlock(
-        default={"phase_presentation": iapws95.PhaseType.LG}
+        default={"phase_presentation": iapws95.PhaseType.MIX}
     )
 
     # A physical property parameter block with temperature, pressure and vapor
@@ -98,7 +102,7 @@ def create_model():
     # providing several configuration options, including: throttle valves;
     # high-, intermediate-, and low-pressure sections; steam extractions; and
     # pressure driven flow.  See the IDAES documentation for details.
-    m.fs.turb = TurbineMultistage(
+    m.fs.turb = HelmTurbineMultistage(
         default={
             "property_package": m.fs.prop_water,
             "num_parallel_inlet_stages": 4,  # number of admission arcs
@@ -129,7 +133,8 @@ def create_model():
 
     # Create a variable for pressure change in the reheater (assuming
     # reheat_delta_p should be negative).
-    m.fs.turb.reheat_delta_p = pyo.Var(m.fs.time, initialize=0)
+    m.fs.turb.reheat_delta_p = pyo.Var(
+        m.fs.time, initialize=0, units=pyo.units.Pa)
 
     # Add a constraint to calculate the IP section inlet pressure based on the
     # pressure drop in the reheater and the outlet pressure of the HP section.
@@ -142,7 +147,8 @@ def create_model():
 
     # Create a variable for reheat temperature and fix it to the desired reheater
     # outlet temperature
-    m.fs.turb.reheat_out_T = pyo.Var(m.fs.time, initialize=866)
+    m.fs.turb.reheat_out_T = pyo.Var(
+        m.fs.time, initialize=866, units=pyo.units.K)
 
     # Create a constraint for the IP section inlet temperature.
     @m.fs.turb.Constraint(m.fs.time)
@@ -157,7 +163,7 @@ def create_model():
     ############################################################################
     # Add a mixer for all the streams coming into the condenser.  In this case the
     # main steam, and the boiler feed pump turbine outlet go to the condenser
-    m.fs.condenser_mix = Mixer(
+    m.fs.condenser_mix = HelmMixer(
         default={
             "momentum_mixing_type": MomentumMixingType.none,
             "inlet_list": ["main", "bfpt"],
@@ -175,76 +181,49 @@ def create_model():
     @m.fs.condenser_mix.Constraint(m.fs.time)
     def mixer_pressure_constraint(b, t):
         return b.main_state[t].pressure == b.mixed_state[t].pressure
+    # PYLINT-TODO the name "mixer_pressure_constraint" is reused below to define other constraint functions,
+    # causing pylint to report function-redefined errors
+    # this likely does not actually cause issues at runtime,
+    # but it could be worth to check anyway if the pylint errors can be addressed
+    # e.g. by giving a unique name to each of the affected functions
+    # pylint: disable=function-redefined
 
     # The condenser model uses the physical property model with TPx state
     # variables, while the rest of the model uses PH state variables. To
     # translate between the two property calculations, an extra port is added to
     # the mixer which contains temperature, pressure, and vapor fraction
     # quantities.
+    m.fs.condenser_mix._flow_mol_ref = pyo.Reference(
+        m.fs.condenser_mix.mixed_state[:].flow_mol)
+    m.fs.condenser_mix._temperature_ref = pyo.Reference(
+        m.fs.condenser_mix.mixed_state[:].temperature)
+    m.fs.condenser_mix._pressure_ref = pyo.Reference(
+        m.fs.condenser_mix.mixed_state[:].pressure)
+    m.fs.condenser_mix._vapor_frac_ref = pyo.Reference(
+        m.fs.condenser_mix.mixed_state[:].vapor_frac)
+
     m.fs.condenser_mix.outlet_tpx = Port(
         initialize={
-            "flow_mol": pyo.Reference(
-                m.fs.condenser_mix.mixed_state[:].flow_mol),
-            "temperature": pyo.Reference(
-                m.fs.condenser_mix.mixed_state[:].temperature),
-            "pressure": pyo.Reference(
-                m.fs.condenser_mix.mixed_state[:].pressure),
-            "vapor_frac": pyo.Reference(
-                m.fs.condenser_mix.mixed_state[:].vapor_frac),
+            "flow_mol": m.fs.condenser_mix._flow_mol_ref,
+            "temperature": m.fs.condenser_mix._temperature_ref,
+            "pressure": m.fs.condenser_mix._pressure_ref,
+            "vapor_frac": m.fs.condenser_mix._vapor_frac_ref,
         }
     )
 
-    # Add the heat exchanger model for the condenser.
-    m.fs.condenser = HeatExchanger(
-        default={
-            "delta_temperature_callback": delta_temperature_underwood_callback,
-            "shell": {"property_package": m.fs.prop_water_tpx},
-            "tube": {"property_package": m.fs.prop_water},
-        }
-    )
-    m.fs.condenser.delta_temperature_out.fix(5)
 
-    # Everything condenses so the saturation pressure determines the condenser
-    # pressure. Deactivate the constraint that is used in the TPx version vapor
-    # fraction constraint and fix vapor fraction to 0.
-    m.fs.condenser.shell.properties_out[:].eq_complementarity.deactivate()
-    m.fs.condenser.shell.properties_out[:].vapor_frac.fix(0)
-
-    # There is some subcooling in the condenser, so we assume the condenser
-    # pressure is actually going to be slightly higher than the saturation
-    # pressure.
-    m.fs.condenser.pressure_over_sat = pyo.Var(
-        m.fs.time,
-        initialize=500,
-        doc="Pressure added to Psat in the condeser. This is to account for"
-        "some subcooling. (Pa)",
-    )
-    # Add a constraint for condenser pressure
-    @m.fs.condenser.Constraint(m.fs.time)
-    def eq_pressure(b, t):
-        return (
-            b.shell.properties_out[t].pressure
-            == b.shell.properties_out[t].pressure_sat + b.pressure_over_sat[t]
-        )
-
-    # Extra port on condenser to hook back up to pressure-enthalpy properties
-    m.fs.condenser._outlet_1_enth_mol_ref = pyo.Reference(
-        m.fs.condenser.shell.properties_out[:].enth_mol
-    )
-    m.fs.condenser.outlet_1_ph = Port(
-        initialize={
-            "flow_mol": pyo.Reference(
-                m.fs.condenser.shell.properties_out[:].flow_mol),
-            "pressure": pyo.Reference(
-                m.fs.condenser.shell.properties_out[:].pressure),
-            "enth_mol": pyo.Reference(
-                m.fs.condenser.shell.properties_out[:].enth_mol),
-        }
-    )
+    # Add NTU condenser model
+    m.fs.condenser = Condenser(
+        default={"dynamic": False,
+                 "shell": {"has_pressure_change": False,
+                           "property_package": m.fs.prop_water},
+                 "tube": {"has_pressure_change": False,
+                          "property_package": m.fs.prop_water}
+                 })
 
     # Add the condenser hotwell.  In steady state a mixer will work.  This is
     # where makeup water is added if needed.
-    m.fs.hotwell = Mixer(
+    m.fs.hotwell = HelmMixer(
         default={
             "momentum_mixing_type": MomentumMixingType.none,
             "inlet_list": ["condensate", "makeup"],
@@ -257,13 +236,9 @@ def create_model():
     def mixer_pressure_constraint(b, t):
         return b.condensate_state[t].pressure == b.mixed_state[t].pressure
 
-    # Condensate pump
-    m.fs.cond_pump = PressureChanger(
-        default={
-            "property_package": m.fs.prop_water,
-            "thermodynamic_assumption": ThermodynamicAssumption.pump,
-        }
-    )
+    # Condensate pump (Use compressor model, since it is more robust if vapor form)
+    m.fs.cond_pump = HelmIsentropicCompressor(
+        default={"property_package": m.fs.prop_water})
     ############################################################################
     #  Add low pressure feedwater heaters                                      #
     ############################################################################
@@ -286,14 +261,10 @@ def create_model():
         }
     )
     # pump for fwh1 condensate, to pump it ahead and mix with feedwater
-    m.fs.fwh1_pump = PressureChanger(
-        default={
-            "property_package": m.fs.prop_water,
-            "thermodynamic_assumption": ThermodynamicAssumption.pump,
-        }
-    )
+    m.fs.fwh1_pump = HelmIsentropicCompressor(
+        default={"property_package": m.fs.prop_water})
     # Mix the FWH1 drain back into the feedwater
-    m.fs.fwh1_return = Mixer(
+    m.fs.fwh1_return = HelmMixer(
         default={
             "momentum_mixing_type": MomentumMixingType.none,
             "inlet_list": ["feedwater", "fwh1_drain"],
@@ -345,7 +316,7 @@ def create_model():
     ############################################################################
     # The deaerator is basically an open tank with multiple inlets.  For steady-
     # state, a mixer model is sufficient.
-    m.fs.fwh5_da = Mixer(
+    m.fs.fwh5_da = HelmMixer(
         default={
             "momentum_mixing_type": MomentumMixingType.none,
             "inlet_list": ["steam", "drain", "feedwater"],
@@ -359,19 +330,11 @@ def create_model():
         return b.feedwater_state[t].pressure == b.mixed_state[t].pressure
 
     # Add the boiler feed pump and boiler feed pump turbine
-    m.fs.bfp = PressureChanger(
-        default={
-            "property_package": m.fs.prop_water,
-            "thermodynamic_assumption": ThermodynamicAssumption.pump,
-        }
-    )
-    m.fs.bfpt = PressureChanger(
-        default={
-            "property_package": m.fs.prop_water,
-            "compressor": False,
-            "thermodynamic_assumption": ThermodynamicAssumption.isentropic,
-        }
-    )
+    m.fs.bfp = HelmIsentropicCompressor(
+        default={"property_package": m.fs.prop_water})
+
+    m.fs.bfpt = HelmTurbineStage(
+        default={"property_package": m.fs.prop_water})
 
     # The boiler feed pump outlet pressure is the same as the condenser
     @m.fs.Constraint(m.fs.time)
@@ -390,7 +353,8 @@ def create_model():
     def constraint_out_enthalpy(b, t):
         return (
             b.bfpt.control_volume.properties_out[t].enth_mol
-            == b.bfpt.control_volume.properties_out[t].enth_mol_sat_phase["Vap"] - 200
+            == b.bfpt.control_volume.properties_out[t].enth_mol_sat_phase["Vap"] -
+            200*pyo.units.J/pyo.units.mol
         )
 
     # The boiler feed pump power is the same as the power generated by the
@@ -499,12 +463,15 @@ def create_model():
     m.fs.EXHST_MAIN = Arc(
         source=m.fs.turb.outlet_stage.outlet, destination=m.fs.condenser_mix.main
     )
+
     m.fs.condenser_mix_to_condenser = Arc(
-        source=m.fs.condenser_mix.outlet_tpx, destination=m.fs.condenser.inlet_1
+        source=m.fs.condenser_mix.outlet, destination=m.fs.condenser.inlet_1
     )
+
     m.fs.COND_01 = Arc(
-        source=m.fs.condenser.outlet_1_ph, destination=m.fs.hotwell.condensate
+        source=m.fs.condenser.outlet_1, destination=m.fs.hotwell.condensate
     )
+
     m.fs.COND_02 = Arc(source=m.fs.hotwell.outlet, destination=m.fs.cond_pump.inlet)
     ############################################################################
     #  Low pressure FWHs                                                       #
@@ -661,8 +628,8 @@ def set_model_input(m):
     #  Turbine input                                                           #
     ############################################################################
     main_steam_pressure = 2.423e7
-    m.fs.turb.turbine_inlet_cf_fix(2.7e-4)
-    m.fs.turb.turbine_outlet_cf_fix(0.15)
+    m.fs.turb.turbine_inlet_cf_fix(4.6e-4)
+    m.fs.turb.turbine_outlet_cf_fix(0.32)
     # Set the turbine steam inlet conditions and flow guess for init
     m.fs.turb.inlet_split.inlet.enth_mol.fix(62710)
     m.fs.turb.inlet_split.inlet.pressure.fix(main_steam_pressure)
@@ -694,20 +661,16 @@ def set_model_input(m):
     ############################################################################
     #  Condenser section inputs                                                #
     ############################################################################
-    # cooling water
     m.fs.condenser.inlet_2.flow_mol.fix(2500000)
     m.fs.condenser.inlet_2.enth_mol.fix(1700)
     m.fs.condenser.inlet_2.pressure.fix(500000)
-    # Condenser
-    m.fs.condenser.pressure_over_sat.fix(100)
-    m.fs.condenser.delta_temperature_out.fix(3)
-    m.fs.condenser.area.fix(2.2e4)
-    m.fs.condenser.area.unfix()  # to make solving easier, specifying approach
-    m.fs.condenser.overall_heat_transfer_coefficient.fix(2000)
+    m.fs.condenser.area.fix(13000)
+    m.fs.condenser.overall_heat_transfer_coefficient.fix(15000)
+
     m.fs.hotwell.makeup.flow_mol[:].value = 1 # don't fix is calculated
     m.fs.hotwell.makeup.enth_mol.fix(2500)
     m.fs.hotwell.makeup.pressure.fix(101325)
-    m.fs.cond_pump.efficiency_pump.fix(0.80)
+    m.fs.cond_pump.efficiency_isentropic.fix(0.80)
     m.fs.cond_pump.deltaP.fix(1e6)
     ############################################################################
     #  Low-pressure FWH section inputs                                         #
@@ -719,7 +682,7 @@ def set_model_input(m):
     m.fs.fwh1.condense.area.fix(400)
     m.fs.fwh1.condense.overall_heat_transfer_coefficient.fix(2000)
     # fwh1 pump
-    m.fs.fwh1_pump.efficiency_pump.fix(0.80)
+    m.fs.fwh1_pump.efficiency_isentropic.fix(0.80)
     m.fs.fwh1_pump.deltaP.fix(1.2e6) # need pressure higher than feedwater
     # fwh2
     m.fs.fwh2.condense.area.fix(150)
@@ -745,7 +708,7 @@ def set_model_input(m):
     ############################################################################
     #  Deaerator and boiler feed pump (BFP) input                              #
     ############################################################################
-    m.fs.bfp.efficiency_pump.fix(0.80)
+    m.fs.bfp.efficiency_isentropic.fix(0.80)
     m.fs.bfp.outlet.pressure[:].value = main_steam_pressure * 1.1 # guess
     m.fs.bfpt.efficiency_isentropic.value = 0.80 #don't fix, just initial guess
     ############################################################################
@@ -790,12 +753,38 @@ def initialize(m, fileinput=None, outlvl=idaeslog.NOTSET):
     """
     init_log = idaeslog.getInitLogger(m.name, outlvl, tag="flowsheet")
     solve_log = idaeslog.getSolveLogger(m.name, outlvl, tag="flowsheet")
-    solver = pyo.SolverFactory("ipopt")
-    solver.options = {
-        "tol": 1e-7,
-        "linear_solver": "ma27",
-        "max_iter": 40,
-    }
+
+    #set scaling factors
+
+    iscale.set_scaling_factor(m.fs.condenser.side_1.heat, 1e-9)
+    iscale.set_scaling_factor(m.fs.condenser.side_2.heat, 1e-9)
+
+    iscale.set_scaling_factor(m.fs.fwh1.condense.side_1.heat, 1e-7)
+    iscale.set_scaling_factor(m.fs.fwh1.condense.side_2.heat, 1e-7)
+
+    iscale.set_scaling_factor(m.fs.fwh2.condense.side_1.heat, 1e-7)
+    iscale.set_scaling_factor(m.fs.fwh2.condense.side_2.heat, 1e-7)
+
+    iscale.set_scaling_factor(m.fs.fwh3.condense.side_1.heat, 1e-7)
+    iscale.set_scaling_factor(m.fs.fwh3.condense.side_2.heat, 1e-7)
+
+    iscale.set_scaling_factor(m.fs.fwh4.condense.side_1.heat, 1e-7)
+    iscale.set_scaling_factor(m.fs.fwh4.condense.side_2.heat, 1e-7)
+
+    iscale.set_scaling_factor(m.fs.fwh6.condense.side_1.heat, 1e-7)
+    iscale.set_scaling_factor(m.fs.fwh6.condense.side_2.heat, 1e-7)
+
+    iscale.set_scaling_factor(m.fs.fwh7.condense.side_1.heat, 1e-7)
+    iscale.set_scaling_factor(m.fs.fwh7.condense.side_2.heat, 1e-7)
+
+    iscale.set_scaling_factor(m.fs.fwh8.condense.side_1.heat, 1e-7)
+    iscale.set_scaling_factor(m.fs.fwh8.condense.side_2.heat, 1e-7)
+
+    iscale.calculate_scaling_factors(m)
+
+
+    solver = get_solver()
+
     if fileinput is not None:
         init_log.info("Loading initial values from file: {}".format(fileinput))
         ms.from_json(m, fname=fileinput)
@@ -821,7 +810,8 @@ def initialize(m, fileinput=None, outlvl=idaeslog.NOTSET):
     # Put in a rough initial guess for the IP section inlet, since it is
     # disconnected from the HP section for the reheater.
     ip1_pin = 5.35e6
-    ip1_hin = iapws95.htpx(T=866, P=ip1_pin)
+    ip1_hin = pyo.value(iapws95.htpx(
+        T=866*pyo.units.K, P=ip1_pin*pyo.units.Pa))
     ip1_fin = pyo.value(m.fs.turb.inlet_split.inlet.flow_mol[0])
     m.fs.turb.ip_stages[1].inlet.enth_mol[:].value = ip1_hin
     m.fs.turb.ip_stages[1].inlet.flow_mol[:].value = ip1_fin
@@ -859,22 +849,14 @@ def initialize(m, fileinput=None, outlvl=idaeslog.NOTSET):
     _set_port(m.fs.condenser_mix.bfpt, m.fs.bfpt.outlet)
     m.fs.condenser_mix.initialize(outlvl=outlvl, optarg=solver.options)
     # initialize condenser hx
-    _set_port(m.fs.condenser.inlet_1, m.fs.condenser_mix.outlet_tpx)
+
+    _set_port(m.fs.condenser.inlet_1, m.fs.condenser_mix.outlet)
     _set_port(m.fs.condenser.outlet_2, m.fs.condenser.inlet_2)
-    # This still has the outlet vapor fraction fixed at 0, so if that isn't
-    # true for the initial pressure guess this could fail to initialize
-    m.fs.condenser.inlet_1.fix()
-    m.fs.condenser.inlet_1.pressure.unfix()
-    with idaeslog.solver_log(solve_log, idaeslog.DEBUG) as slc:
-        res = solver.solve(m.fs.condenser, tee=slc.tee)
-    init_log.info("Condenser initialized: {}".format(idaeslog.condition(res)))
-    init_log.info_high("Init condenser pressure: {}".format(
-            m.fs.condenser.shell.properties_in[0].pressure.value
-        )
-    )
-    m.fs.condenser.inlet_1.unfix()
+    m.fs.condenser.initialize(outlvl=outlvl, optarg=solver.options)
+
     # initialize hotwell
-    _set_port(m.fs.hotwell.condensate, m.fs.condenser.outlet_1_ph)
+    _set_port(m.fs.hotwell.condensate, m.fs.condenser.outlet_1)
+
     m.fs.hotwell.initialize(outlvl=outlvl, optarg=solver.options)
     m.fs.hotwell.condensate.unfix()
     # initialize condensate pump
@@ -977,7 +959,7 @@ def pfd_result(m, df, svg):
         tags[i + "_T"] = df.loc[i, "T (K)"]
         tags[i + "_P"] = df.loc[i, "P (Pa)"]
         tags[i + "_X"] = df.loc[i, "Vapor Fraction"]
-    # Add some addtional quntities from the model to report
+    # Add some additional quntities from the model to report
     tags["gross_power"] = -pyo.value(m.fs.turb.power[0])
     tags["gross_power_mw"] = -pyo.value(m.fs.turb.power[0]) * 1e-6
     tags["steam_mass_flow"] = df.loc["STEAM_MAIN", "Mass Flow (kg/s)"]
@@ -986,7 +968,7 @@ def pfd_result(m, df, svg):
     tags["steam_pressure"] = df.loc["STEAM_MAIN", "P (Pa)"] / 1000.0
     tags["cond_pressure"] = df.loc["EXHST_MAIN", "P (Pa)"] / 1000.0
     tags["bfp_power"] = pyo.value(m.fs.bfp.work_mechanical[0])
-    tags["bfp_eff"] = pyo.value(m.fs.bfp.efficiency_pump[0]) * 100
+    tags["bfp_eff"] = pyo.value(m.fs.bfp.efficiency_isentropic[0]) * 100
     tags["bfpt_power"] = pyo.value(m.fs.bfpt.work_mechanical[0])
     tags["bfpt_eff"] = pyo.value(m.fs.bfpt.efficiency_isentropic[0]) * 100
 
