@@ -1,3 +1,15 @@
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
+# Lawrence Berkeley National Laboratory,  National Technology & Engineering
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
+#
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
 from idaes.surrogate import alamopy
 from idaes.surrogate.pysmo import polynomial_regression, radial_basis_function, kriging
 from idaes.surrogate.my_surrogate_base import Surrogate
@@ -32,8 +44,6 @@ class GeneralSurrogate(Surrogate):
     CONFIG.declare('logexp', ConfigValue(default=None, domain=bool))
     CONFIG.declare('sincos', ConfigValue(default=None, domain=bool))
     CONFIG.declare('ratio', ConfigValue(default=False, domain=bool))
-    CONFIG.declare('kriging', ConfigValue(default=False, domain=bool))
-    CONFIG.declare('rbf', ConfigValue(default=False, domain=bool))
     CONFIG.declare('alamo_modeler', ConfigValue(default=1, domain=int))
     CONFIG.declare('convpen', ConfigValue(default=0, domain=int))
     CONFIG.declare('basis_function', ConfigValue(default=None, domain=str))
@@ -43,6 +53,13 @@ class GeneralSurrogate(Surrogate):
     CONFIG.declare('overwrite', ConfigValue(default=True, domain=bool))
     CONFIG.declare('fname', ConfigValue(default=None, domain=str))
 
+    CONFIG.declare('alamopy', ConfigValue(default=True, domain=bool))
+    CONFIG.declare('alamopy_rbf', ConfigValue(default=False, domain=bool))
+    CONFIG.declare('pysmo_kriging', ConfigValue(default=False, domain=bool))
+    CONFIG.declare('pysmo_rbf', ConfigValue(default=False, domain=bool))
+    CONFIG.declare('pysmo_polyregression', ConfigValue(default=True, domain=bool))
+
+
     def __init__(self, **settings):
         super().__init__(**settings)
 
@@ -57,37 +74,39 @@ class GeneralSurrogate(Surrogate):
         super().build_model()
 
         self.parse_config()
-        models = []
+        # models = []
 
         modeler_krig = Pysmo_kriging(**self._pysmo_krg_settings)
         modeler_krig.regressed_data(self._rdata_in, self._rdata_out)
-        modeler_krig.build_model()
-        if self.config['kriging']:
+        if self.config['pysmo_kriging']:
+            modeler_krig.build_model()
             self._models.append(modeler_krig)
 
         modeler_rbf = Pysmo_rbf(**self._pysmo_rbf_settings)
         modeler_rbf.regressed_data(self._rdata_in, self._rdata_out)
-        modeler_rbf.build_model()
-        if self.config['rbf']:
+        if self.config['pysmo_rbf']:
+            modeler_rbf.build_model()
             self._models.append(modeler_rbf)
 
         modeler_pr = Pysmo_polyregression(**self._pysmo_pr_settings)
         modeler_pr.regressed_data(self._rdata_in, self._rdata_out)
-        modeler_pr.build_model()
-        self._models.append(modeler_pr)
+        if self.config['pysmo_polyregression']:
+            modeler_pr.build_model()
+            self._models.append(modeler_pr)
 
         modeler_alamo = Alamopy(**self._alamo_settings)
         modeler_alamo.regressed_data(self._rdata_in, self._rdata_out)
-        has_alamo_flag = alamopy.multos.has_alamo()
-        if has_alamo_flag:
-            modeler_alamo.build_model()
-            self._models.append(modeler_alamo)
-        else:
-            print("ALAMO is not installed, and wasn't used for the General Surrogate.")
+        if self.config['alamopy']:
+            has_alamo_flag = alamopy.multos.has_alamo()
+            if has_alamo_flag:
+                modeler_alamo.build_model()
+                self._models.append(modeler_alamo)
+            else:
+                print("ALAMO is not installed, and wasn't used for the General Surrogate.")
 
         _alamo_settings_rbf = self._alamo_settings
-        if self.config['rbf']:
-            _alamo_settings_rbf['grbfcns'] = self.config['rbf']
+        if self.config['alamopy_rbf']:
+            _alamo_settings_rbf['grbfcns'] = self.config['alamopy_rbf']
             modeler_alamo_rbf = Alamopy(**_alamo_settings_rbf)
             modeler_alamo_rbf.regressed_data(self._rdata_in, self._rdata_out)
             if has_alamo_flag:
@@ -105,12 +124,12 @@ class GeneralSurrogate(Surrogate):
                 self._results = m.get_results()
                 self._model = m.get_model()
 
-        if not self.config['kriging']:
-            self._models.append(modeler_krig)
-
-        if not self.config['rbf']:
+        if not self.config['pysmo_rbf']:
             self._models.append(modeler_rbf)
 
+        if not self.config['pysmo_kriging']:
+            self._models.append(modeler_krig)
+
         self.pkl_info['Run settings'] = best_config
         self.pkl_info['Results'] = self._results
         self.pkl_info['Expression'] = self._model
@@ -187,6 +206,7 @@ class Alamopy(Surrogate):
     CONFIG.declare('convpen', ConfigValue(default=None, domain=float))
     CONFIG.declare('screener', ConfigValue(default=None, domain=int))  # not sure
     CONFIG.declare('almname', ConfigValue(default=None, domain=str))
+    CONFIG.declare('showalm', ConfigValue(default=False, domain=bool))
     CONFIG.declare('savescratch', ConfigValue(default=None, domain=str))
     CONFIG.declare('savetrace', ConfigValue(default=None, domain=str))
     CONFIG.declare('expandoutput', ConfigValue(default=True, domain=bool))
@@ -216,18 +236,25 @@ class Alamopy(Surrogate):
         self.pkl_info['Results'] = self._results
         self.pkl_info['Expression'] = self._model
 
-    # JSON
-    # self.pkl_info['Expression'] = self.alamopy_results['model']
-
     def handle_results(self, res):
         # sympy to pyomo converter
 
-        self._results[Metrics.RMSE] = self.alamopy_results['rmse']
-        self._results[Metrics.SSE] = self.alamopy_results['ssr']
-        self._results[Metrics.Time] = self.alamopy_results['totaltime']
-        self._results[Metrics.MSE] = float(self.alamopy_results['rmse']) ** 2
-        self._results[Metrics.Order] = self.alamopy_results['nbas']
-        self._results[Metrics.R2] = self.alamopy_results['R2']
+        self._results = {}
+        if type(self.alamopy_results[Metrics.R2]) is dict:
+            label = self.alamopy_results['zlabels'][0]
+            self._results[Metrics.RMSE] = self.alamopy_results['rmse'][label]
+            self._results[Metrics.SSE] = self.alamopy_results['ssr'][label]
+            self._results[Metrics.Time] = self.alamopy_results['totaltime'][label]
+            self._results[Metrics.MSE] = float(self.alamopy_results['rmse'][label]) ** 2
+            self._results[Metrics.Order] = self.alamopy_results['nbas'][label]
+            self._results[Metrics.R2] = self.alamopy_results['R2'][label]
+        else:
+            self._results[Metrics.RMSE] = self.alamopy_results['rmse']
+            self._results[Metrics.SSE] = self.alamopy_results['ssr']
+            self._results[Metrics.Time] = self.alamopy_results['totaltime']
+            self._results[Metrics.MSE] = float(self.alamopy_results['rmse']) ** 2
+            self._results[Metrics.Order] = self.alamopy_results['nbas']
+            self._results[Metrics.R2] = self.alamopy_results['R2']
 
         # Generate pyomo expression
         m = pyo.ConcreteModel()
@@ -391,7 +418,7 @@ class Pysmo_rbf(Surrogate):
         self.rbf_setup = {k: self.config[k] for k in set(self.config) - set(self.pyomo_vars)}
         prob_init = radial_basis_function.RadialBasisFunctions(training_data, **self.rbf_setup)
         feature_vec = prob_init.get_feature_vector()
-        self.pysmo_rbf_results = prob_init.rbf_training()
+        self.pysmo_rbf_results = prob_init.training()
         self._results[Metrics.Time] = time.time() - start_time
 
         self.handle_results(feature_vec)
@@ -408,11 +435,14 @@ class Pysmo_rbf(Surrogate):
         self._results[Metrics.R2] = self.pysmo_rbf_results.R2
         # Generate Pyomo expression
         if self.pyomo_vars:
-            self._model = self.pysmo_rbf_results.rbf_generate_expression(self.pyomo_vars['pyomo_vars'])
+            self._model = self.pysmo_rbf_results.generate_expression(self.pyomo_vars['pyomo_vars'])
         else:
             list_vars = []
             for i in feature_vec.keys():
                 list_vars.append(feature_vec[i])
+            # PYLINT-TODO-FIX the method "rbf_generate_expression" does not exist for this class
+            # maybe the name of the method should be "generate_expression" like in the self.pyomo_vars case?
+            # pylint: disable=no-member
             self._model = self.pysmo_rbf_results.rbf_generate_expression(list_vars)
 
 
@@ -438,7 +468,7 @@ class Pysmo_kriging(Surrogate):
         self.krg_setup = {k: self.config[k] for k in set(self.config) - set(self.pyomo_vars)}
         prob_init = kriging.KrigingModel(training_data, **self.krg_setup)
         feature_vec = prob_init.get_feature_vector()
-        self.pysmo_kriging_results = prob_init.kriging_training()
+        self.pysmo_kriging_results = prob_init.training()
         self._results[Metrics.Time] = time.time() - start_time
 
         self.handle_results(feature_vec)
@@ -459,11 +489,14 @@ class Pysmo_kriging(Surrogate):
         for i in feature_vec.keys():
             list_vars.append(feature_vec[i])
         if self.pyomo_vars:
-            self._model = self.pysmo_kriging_results.kriging_generate_expression(self.pyomo_vars['pyomo_vars'])
+            self._model = self.pysmo_kriging_results.generate_expression(self.pyomo_vars['pyomo_vars'])
         else:
             list_vars = []
             for i in feature_vec.keys():
                 list_vars.append(feature_vec[i])
+            # PYLINT-TODO-FIX the method "kriging_generate_expression" does not exist for this class
+            # maybe the name of the method should be "generate_expression" like in the self.pyomo_vars case?
+            # pylint: disable=no-member
             self._model = self.pysmo_kriging_results.kriging_generate_expression(list_vars)
 
 
@@ -504,7 +537,7 @@ class Pysmo_polyregression(Surrogate):
             except:
                 raise ValueError('The additional terms could not be evaluated.')
 
-        self.pysmo_polyregression_results = prob_init.poly_training()
+        self.pysmo_polyregression_results = prob_init.training()
         self._results[Metrics.Time] = time.time() - start_time
 
         self.handle_results(feature_vec)
