@@ -1,93 +1,31 @@
-##############################################################################
-# Institute for the Design of Advanced Energy Systems Process Systems
-# Engineering Framework (IDAES PSE Framework) Copyright (c) 2018-2020, by the
-# software owners: The Regents of the University of California, through
+#################################################################################
+# The Institute for the Design of Advanced Energy Systems Integrated Platform
+# Framework (IDAES IP) was produced under the DOE Institute for the
+# Design of Advanced Energy Systems (IDAES), and is copyright (c) 2018-2021
+# by the software owners: The Regents of the University of California, through
 # Lawrence Berkeley National Laboratory,  National Technology & Engineering
-# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia
-# University Research Corporation, et al. All rights reserved.
+# Solutions of Sandia, LLC, Carnegie Mellon University, West Virginia University
+# Research Corporation, et al.  All rights reserved.
 #
-# Please see the files COPYRIGHT.txt and LICENSE.txt for full copyright and
-# license information, respectively. Both files are also available online
-# at the URL "https://github.com/IDAES/idaes-pse".
-##############################################################################
+# Please see the files COPYRIGHT.md and LICENSE.md for full copyright and
+# license information.
+#################################################################################
+
+__author__ = "Oluwamayowa Amusat"
+
+# Imports from the python standard library
+import os.path
+import pprint
+# Imports from third parties
+from matplotlib import pyplot as plt
 import numpy as np
+import pandas as pd
+import pickle
+from pyomo.core import Param, exp
 from scipy.optimize import basinhopping
 import scipy.optimize as opt
-import pandas as pd
-from pyomo.core import Param, exp
+# Imports from IDAES namespace
 from idaes.surrogate.pysmo.sampling import FeatureScaling as fs
-import os.path, pickle
-from matplotlib import pyplot as plt
-
-
-class ResultReport:
-
-    def __init__(self, theta, reg_param, mean, variance, cov_mat, cov_inv, ymu, y_training, r2_training, rmse_error, p, x_data, x_data_scaled, x_data_min, x_data_max):
-        """
-        A class for creating an object containing information about the Kriging model to be returned to the user.
-
-            Returns:
-            self function containing several attributes -
-
-                self.optimal_weights(NumPy Array)          : array containing kriging hyperparameters (coefficients)
-                self.regularization_parameter(float)      : best regularization parameter found.
-                self.optimal_mean(float)                  : Kriging model mean
-                self.optimal_variance(float)              : Kriging model variance
-                self.optimal_covariance_matrix(NumPy Array): Kriging model covaruiance matrix (regularized)
-                self.covariance_matrix_inverse(NumPy Array): Inverse of the Kriging covariance matrix
-                self.output_predictions(NumPy Array)       : Output predictions fosupplied inputs based on final Kriging model
-                self.training_R2                            : R2 coefficient-of-fit between the actual output and the surrogate predictions
-                self.training_rmse                          : RMSE error on the training output predictions
-                self.x_data(NumPy Array)                   : Input features data (unscaled)
-                self.scaled_x(NumPy Array)                 : Input features data (scaled)
-                self.x_min = x_data_min                     : Lower bounds of features data
-                self.x_max = x_data_max                     : Upper bounds of features data
-
-        """
-        self.optimal_weights = theta
-        self.optimal_p = p
-        self.optimal_mean = mean
-        self.optimal_variance = variance
-        self.regularization_parameter = reg_param
-        self.optimal_covariance_matrix = cov_mat
-        self.covariance_matrix_inverse = cov_inv
-        self.optimal_y_mu = ymu
-        self.output_predictions = y_training
-        self.training_R2 = r2_training
-        self.training_rmse = rmse_error
-        self.x_data = x_data
-        self.scaled_x = x_data_scaled
-        self.x_min = x_data_min
-        self.x_max = x_data_max
-
-    def kriging_generate_expression(self, variable_list):
-        """
-        The ``kriging_generate_expression`` method returns the Pyomo expression for the Kriging model trained.
-
-        The expression is constructed based on the supplied list of variables **variable_list** and the results of the previous Kriging training process.
-
-        Args:
-            variable_list(list)           : List of input variables to be used in generating expression. This can be the a list generated from the output of ``get_feature_vector``.  The user can also choose to supply a new list of the appropriate length.
-
-        Returns:
-            Pyomo Expression              : Pyomo expression of the Kriging model based on the variables provided in **variable_list**
-
-        """
-        t1 = np.array([variable_list])
-        phi_var = []
-        for i in range(0, self.x_data.shape[0]):
-            curr_term = sum(self.optimal_weights[j] * ( ((t1[0, j] - self.x_min[0, j])/(self.x_max[0, j] - self.x_min[0, j])) - self.scaled_x[i, j]) ** self.optimal_p for j in range(0, self.x_data.shape[1]))
-
-            curr_term = exp(-curr_term)
-            phi_var.append(curr_term)
-        phi_var_array = np.asarray(phi_var)
-
-        phi_inv_times_y_mu = np.matmul(self.covariance_matrix_inverse, self.optimal_y_mu)
-        phi_inv_times_y_mu = phi_inv_times_y_mu.reshape(phi_inv_times_y_mu.shape[0], )
-        kriging_expr = self.optimal_mean[0,0]
-        kriging_expr += sum(w * t for w, t in zip(np.nditer(phi_inv_times_y_mu), np.nditer(phi_var_array, flags=['refs_ok']) ))
-        return kriging_expr
-
 
 class MyBounds(object):
     """
@@ -110,7 +48,7 @@ class KrigingModel:
     """
     The KrigingModel class trains a Kriging model for a training data set.
 
-    The class must first be initialized by calling **KrigingModel**. Model training is then carried out by calling the ``kriging_training`` method.
+    The class must first be initialized by calling **KrigingModel**. Model training is then carried out by calling the ``training`` method.
 
     **KrigingModel** is able to generate either an interpolating or a regressing Kriging model depending on the settings used during initialization..
 
@@ -123,8 +61,8 @@ class KrigingModel:
         >>> p = d.get_feature_vector()
         
         # Train Kriging model and predict output for an test data x_test
-        >>> results = d.kriging_training()
-        >>> predictions = d.kriging_predict_output(results, x_test)
+        >>> d.training()
+        >>> predictions = d.predict_output(x_test)
 
     Args:
         XY_data (NumPy Array or Pandas Dataframe)   : The dataset for Kriging training. **XY_data** is expected to contain both the features (X) and output (Y) information, with the output values (Y) in the last column.
@@ -212,6 +150,19 @@ class KrigingModel:
         else:
             raise Exception('Choice of regularization must be boolean.')
 
+        # Results
+        self.optimal_weights = None
+        self.optimal_p = None
+        self.optimal_mean = None
+        self.optimal_variance = None
+        self.regularization_parameter = None
+        self.optimal_covariance_matrix = None
+        self.covariance_matrix_inverse = None
+        self.optimal_y_mu = None
+        self.output_predictions = None
+        self.training_R2 = None
+        self.training_rmse = None
+
 
     @staticmethod
     def covariance_matrix_generator(x, theta, reg_param, p):
@@ -405,7 +356,13 @@ class KrigingModel:
         if self.num_grads:
             print('Optimizing kriging parameters using L-BFGS-B algorithm...')
             other_args = (self.x_data_scaled, self.y_data, p)
-            opt_results = opt.minimize(self.objective_function, initial_value, args=other_args, method='L-BFGS-B', jac=self.numerical_gradient, bounds=bounds, options={'gtol': 1e-7}) #, 'disp': True})
+            #opt_results = opt.minimize(self.objective_function, initial_value, args=other_args, method='L-BFGS-B', jac=self.numerical_gradient, bounds=bounds, options={'gtol': 1e-7}) #, 'disp': True})
+            opt_results1 = opt.minimize(self.objective_function, initial_value, args=other_args, method='tnc', jac=self.numerical_gradient, bounds=bounds, options={'gtol': 1e-7})
+            opt_results2 = opt.minimize(self.objective_function, initial_value, args=other_args, method='L-BFGS-B', jac=self.numerical_gradient, bounds=bounds, options={'gtol': 1e-7})  # , 'disp': True})
+            if opt_results1.fun < opt_results2.fun:
+                opt_results = opt_results1
+            else:
+                opt_results = opt_results2
         else:
             print('Optimizing Kriging parameters using Basinhopping algorithm...')
             other_args = {"args": (self.x_data_scaled, self.y_data, p), 'bounds': bounds}
@@ -502,12 +459,11 @@ class KrigingModel:
         r_square = 1 - (ss_residual / ss_total)
         return r_square
 
-    def kriging_predict_output(self, kriging_params, x_pred):
+    def predict_output(self, x_pred):
         """
-        The ``kriging_predict_output`` method generates output predictions for input data x_pred based a previously trained Kriging model.
+        The ``predict_output`` method generates output predictions for input data x_pred based a previously trained Kriging model.
 
         Args:
-            kriging_params (tuple)          : Results of Kriging training generated by calling the ``kriging_training`` method.
             x_pred(NumPy Array)             : Array of designs for which the output is to be evaluated/predicted.
 
         Returns:
@@ -520,12 +476,12 @@ class KrigingModel:
             x_pred = x_pred.reshape(1, len(x_pred))
         y_pred = np.zeros((x_pred.shape[0], 1))
         for i in range(0, x_pred.shape[0]):
-            cmt = (np.matmul(((np.abs(x_pred[i, :] - self.x_data_scaled)) ** kriging_params.optimal_p), kriging_params.optimal_weights)).transpose()
+            cmt = (np.matmul(((np.abs(x_pred[i, :] - self.x_data_scaled)) ** self.optimal_p), self.optimal_weights)).transpose()
             cov_matrix_tests = np.exp(-1 * cmt)
-            y_pred[i, 0] = kriging_params.optimal_mean + np.matmul(np.matmul(cov_matrix_tests.transpose(), kriging_params.covariance_matrix_inverse), kriging_params.optimal_y_mu)
+            y_pred[i, 0] = self.optimal_mean + np.matmul(np.matmul(cov_matrix_tests.transpose(), self.covariance_matrix_inverse), self.optimal_y_mu)
         return y_pred
 
-    def kriging_training(self):
+    def training(self):
         """
         Main function for Kriging training.
 
@@ -536,7 +492,7 @@ class KrigingModel:
             (4) A results object is generated by calling the ``ResultsReport`` class.
 
         Returns:
-            tuple   : Python object (**results**) containing the all information about the best Kriging model obtained, including:
+            tuple   : self object (**results**) containing the all information about the best Kriging model obtained, including:
                 - the Kriging model hyperparameters  (**results.optimal_weights**),
                 - when relevant, the optimal regularization parameter found :math:`\lambda` (**results.regularization_parameter**),
                 - the Kriging mean (**results.optimal_mean**),
@@ -558,10 +514,49 @@ class KrigingModel:
         # Training performance
         training_ss_error, rmse_error, y_training_predictions = self.error_calculation(optimal_theta, p, optimal_mean, opt_cov_inv, optimal_ymu, self.x_data_scaled, self.y_data)
         r2_training = self.r2_calculation(self.y_data, y_training_predictions)
+
         # Return results
-        results = ResultReport(optimal_theta, optimal_reg_param, optimal_mean, optimal_variance, optimal_cov_mat, opt_cov_inv, optimal_ymu, y_training_predictions, r2_training, rmse_error, p, self.x_data, self.x_data_scaled, self.x_data_min, self.x_data_max)
-        self.pickle_save({'set-up':self, 'result':results})
-        return results
+        self.optimal_weights = optimal_theta
+        self.optimal_p = p
+        self.optimal_mean = optimal_mean
+        self.optimal_variance = optimal_variance
+        self.regularization_parameter = optimal_reg_param
+        self.optimal_covariance_matrix = optimal_cov_mat
+        self.covariance_matrix_inverse = opt_cov_inv
+        self.optimal_y_mu = optimal_ymu
+        self.output_predictions = y_training_predictions
+        self.training_R2 = r2_training
+        self.training_rmse = rmse_error
+        self.pickle_save({'model' : self})
+        return self
+
+    def generate_expression(self, variable_list):
+        """
+        The ``generate_expression`` method returns the Pyomo expression for the Kriging model trained.
+
+        The expression is constructed based on the supplied list of variables **variable_list** and the results of the previous Kriging training process.
+
+        Args:
+            variable_list(list)           : List of input variables to be used in generating expression. This can be the a list generated from the output of ``get_feature_vector``.  The user can also choose to supply a new list of the appropriate length.
+
+        Returns:
+            Pyomo Expression              : Pyomo expression of the Kriging model based on the variables provided in **variable_list**
+
+        """
+        t1 = np.array([variable_list])
+        phi_var = []
+        for i in range(0, self.x_data.shape[0]):
+            curr_term = sum(self.optimal_weights[j] * ( ((t1[0, j] - self.x_data_min[0, j])/(self.x_data_max[0, j] - self.x_data_min[0, j])) - self.x_data_scaled[i, j]) ** self.optimal_p for j in range(0, self.x_data.shape[1]))
+
+            curr_term = exp(-curr_term)
+            phi_var.append(curr_term)
+        phi_var_array = np.asarray(phi_var)
+
+        phi_inv_times_y_mu = np.matmul(self.covariance_matrix_inverse, self.optimal_y_mu)
+        phi_inv_times_y_mu = phi_inv_times_y_mu.reshape(phi_inv_times_y_mu.shape[0], )
+        kriging_expr = self.optimal_mean[0,0]
+        kriging_expr += sum(w * t for w, t in zip(np.nditer(phi_inv_times_y_mu), np.nditer(phi_var_array, flags=['refs_ok']) ))
+        return kriging_expr
 
     def get_feature_vector(self):
         """
@@ -593,7 +588,7 @@ class KrigingModel:
     @staticmethod
     def pickle_load(solution_file):
         """
-        pickle_load loads the results of a saved run 'file.obj'. It returns an array of two elements: the setup (index[0]) and the results (index[1]).
+        pickle_load loads the results of a saved run 'file.obj'.])
 
         Input arguments:
                 solution_file            : Pickle object file containing previous solution to be loaded.
@@ -605,11 +600,10 @@ class KrigingModel:
         except:
             raise Exception('File could not be loaded.')
 
-    def parity_residual_plots(self, results_vector):
+    def parity_residual_plots(self):
         """
 
         inputs:
-            results_vector: Result object created by run.
 
         Returns:
 
@@ -618,13 +612,13 @@ class KrigingModel:
         fig1 = plt.figure(figsize=(16, 9), tight_layout=True)
         ax = fig1.add_subplot(121)
         ax.plot(self.y_data, self.y_data, '-')
-        ax.plot(self.y_data, results_vector.output_predictions, 'o')
+        ax.plot(self.y_data, self.output_predictions, 'o')
         ax.set_xlabel(r'True data', fontsize=12)
         ax.set_ylabel(r'Surrogate values', fontsize=12)
         ax.set_title(r'Parity plot', fontsize=12)
 
         ax2 = fig1.add_subplot(122)
-        ax2.plot(self.y_data, self.y_data - results_vector.output_predictions, 's', mfc='w', mec='m', ms=6)
+        ax2.plot(self.y_data, self.y_data - self.output_predictions, 's', mfc='w', mec='m', ms=6)
         ax2.axhline(y=0, xmin=0, xmax=1)
         ax2.set_xlabel(r'True data', fontsize=12)
         ax2.set_ylabel(r'Residuals', fontsize=12)
@@ -634,41 +628,40 @@ class KrigingModel:
 
         return
 
-    def _report(self, results_vector):
+    def _report(self):
         ## Will only work with Python > 3.5
         variable_headers = self.get_feature_vector()
         var_list = []
         for i in variable_headers:
             var_list.append(variable_headers[i])
-        eqn = results_vector.kriging_generate_expression(var_list)
+        eqn = self.generate_expression(var_list)
 
         double_line = "=" * 120
         s = (f"\n{double_line}"
              f"\nResults of Kriging run:\n"
-             f"\nKriging mean                     : {results_vector.optimal_mean}\n"
-             f"Kriging variance                 : {results_vector.optimal_variance}\n"
-             f"Kriging weights                  : {results_vector.optimal_weights}\n"
-             f"Regularization parameter         : {results_vector.regularization_parameter}\n"
-             f"Number of terms in Kriging model : {results_vector.optimal_y_mu.size + 1}\n"
+             f"\nKriging mean                     : {self.optimal_mean}\n"
+             f"Kriging variance                 : {self.optimal_variance}\n"
+             f"Kriging weights                  : {self.optimal_weights}\n"
+             f"Regularization parameter         : {self.regularization_parameter}\n"
+             f"Number of terms in Kriging model : {self.optimal_y_mu.size + 1}\n"
              f"\nKriging Expression:\n"
              f"--------------------\n"
              f"\n{eqn}\n"
              f"--------------------------\n"
              f"\nModel training errors:"
              f"\n-----------------------\n"
-             f"Mean Squared Error (MSE)         : {results_vector.training_rmse ** 2}\n"
-             f"Root Mean Squared Error (RMSE)   : {results_vector.training_rmse}\n"
-             f"Goodness of fit (R2)             : {results_vector.training_R2}\n"
+             f"Mean Squared Error (MSE)         : {self.training_rmse ** 2}\n"
+             f"Root Mean Squared Error (RMSE)   : {self.training_rmse}\n"
+             f"Goodness of fit (R2)             : {self.training_R2}\n"
              f"\n{double_line}"
              )
         return(s)
 
-    def print_report(self, results_vector):
-        s = self._report(results_vector)
+    def print_report(self):
+        s = self._report()
         print(s)
 
-    def _repr_pretty_(self, results_vector):
-        import pprint
-        s = self._report(results_vector)
-        j = pprint.PrettyPrinter(width=80)
-        j.pprint(s)
+    def _repr_pretty_(self, p, cycle=False):
+        s = self._report()
+        p.text(s)
+
